##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Glen Jenness
# Last updated: 2013-07-01
##############################################################
import sys, os, fileinput
import numpy as np

from MAST.utility import MASTError

class GapPlot:
    def __init__(self, gap=0.0, threshold=None, bins=300, dfe=None, real_gap=None):
        self.gap = gap
        self.threshold = threshold
        self.bins = bins
        self.dfe = dfe
        self.real_gap = real_gap

        self.states = list()
        self.transitions = list()

    def add_charge_state(self, charge, dfe):
        self.states.append( [charge, dfe] )

    def get_transition(self, state1, state2):
        #print state1, state2
        label = '(%i/%i)' % (state1[0], state2[0])
        trans = (state2[1] - state1[1]) / abs(state1[0] - state2[0])
        return label, trans

    def plot_levels(self,append=""):
        """Plot defect levels
            Args:
                append <str>: String to append to file name (optional)
        """
        step = self.gap / self.bins

        if (self.threshold is None):
            self.threshold = step

        for condition in self.dfe:
            print 'Plotting levels for %s conditions' % condition
            for defect in self.dfe[condition]:
                print 'Analyzing levels for defect %s.' % defect
                state = self.dfe[condition][defect]
                data = self.plot_defect(state, step)

                print 'Writing energies and grid to file %s-%s%s.txt' % (defect, condition.replace(' ', '_'), append)
                with open('%s-%s%s.txt' % (defect, condition.replace(' ', '_'), append), 'w') as datafile:
                    for datum in data:
                        datafile.write('%10.5f%10.5f\n' % (datum[0], datum[1]))

                print str()

    def plot_defect(self, state, step):
        # Start by finding the lost energy charge state
        sorted_states = sorted(state, key=lambda dfe: dfe[1])

        data = list()
        level = 0.0
        nsteps = 0
        energy = sorted_states[0][1]


        if (len(sorted_states) == 1):
            final_state = sorted_states[0]
        else:
            for i in range(0, len(sorted_states)-1):
                state1 = sorted_states[i]
                state2 = sorted_states[i+1]
                transition = self.get_transition(state1, state2)
                #print transition, state1, state2
                if state2[0] > state1[0]:
                    print 'The transition %s at %.2f is losing electrons as Fermi level increases.' % (transition[0], transition[1])
                    #print 'This is unphysical, please check your DFE\'s and run again.'
                    print 'Skipping this transition'
                    #raise MASTError(self.__class__.__name__, 'Unphysical defect transition levels')
                    # Since state2 is higher in energy, we neglect here, and continue on from state1
                    final_state = state1
                    sorted_states[i+1] = state1 # restart the checking from the current state 
                else:
                    final_state = state2
                    switch = False
                    while not switch:
                        #if (abs(energy - state2[1]) <= self.threshold):
                        if (abs(level - transition[1]) < self.threshold):
                            print 'Transition %s found at %4.2f eV.' % (transition[0], transition[1])
                            #print 'DFE = %4.2f eV' % energy
                            #print transition[1], energy
                            switch = True
                        elif (level > self.gap):
                            switch = True # Level cannot be higher than the gap, so truncate it here
                        else:
                            #print '%10.5f%10.5f%10.5f' % (energy - state2[1], level, energy)

                            if not self.real_gap:
                                data.append( [level, energy] )
                            else:
                                slevel = level * (self.real_gap / self.gap)

                                #print 'Scaling by %f' % (self.real_gap / self.gap)
                                #print 'Old: %f, %f' % (level, energy)
                                #print 'Scaled: %f, %f' % (slevel, energy)
                                data.append( [slevel, energy] )

                            energy += state1[0] * step
                            level += step
                            nsteps += 1

        #final_state = sorted_states[-1]
        while (level <= self.gap):
            if not self.real_gap:
                data.append( [level, energy] )
            else:
                slevel = level * (self.real_gap / self.gap)
                data.append([slevel, energy])

            energy += final_state[0] * step
            level += step

        return data



'''General fitness functions for use in optimizer'''
# from chempotswap import *
# from enpafit import *
# from enthalpyfit import *
# from exponentialfit import *
# from formationenergy import *
# from stem_cost import *
# from surfaceenergy import *
# from totalenfit import *
# from stem_cost_rotation import *
# from sibias import *
# from sibias2 import *


import numpy
import math
from ase import Atom, Atoms
from MAST.structopt.fingerprinting import dirac

def get_fingerprint(Optimizer,indiv,binsize,cutoffdist):
    """Function to calculate the fingerprint of a structure
    """
    rs=numpy.linspace(0.0,cutoffdist,cutoffdist/binsize)
    indi=indiv[0]
    Vuc=indi.get_volume()
    if Optimizer.structure=='Defect':
        solid=Atoms()
        solid.extend(indi)
        solid.extend(indiv.bulki)
    elif Optimizer.structure=='Crystal':
        solid=indi.repeat([3,3,3])
    else:
        solid=indi.copy()
    syms = sorted(list(set([atm.symbol for atm in solid])))
    fingerprints=[]
    for i in range(len(syms)):
        for j in range(i,len(syms)):
            indl=[atm for atm in indi if atm.symbol==syms[i]]
            ind=Atoms()
            for one in indl:
                ind.append(one)
            soll=[atm for atm in solid if atm.symbol==syms[j]]
            sol=Atoms()
            for one in soll:
                sol.append(one)
            soli=[atm for atm in solid if atm.symbol==syms[i]]
            value=[]
            for R in rs:
                value2=[]
                for k in range(len(ind)):
                    value1=[]
                    for m in range(len(sol)):
                        if k !=m:
                            rij=sol.get_distance(k,m,mic=True)
                            if rij==0: 
                                pass
                                #pdb.set_trace()
                            value1.append(dirac(R,a=rij,sig=0.02)*1/(4*math.pi*rij**2*binsize*len(soli)*len(sol)/Vuc))
                    value2.append(sum(value1))
                value.append(sum(value2))
            fingerprints.append(value)
    fpt=[]
    for one in fingerprints:
        fpt.extend(one)
    return fpt




import random
import numpy
from ase import Atom, Atoms

def read_Ltraj_file(filename,atomlist=False,timestep=-1,atomlist=False,ratomlist=False,writefile=False):
    """Function to convert LAMMPS Trajectory file to ASE atoms object
    Inputs:
        filename = string of LAMMPS Trajectory file to read
        timestep = Which timestep to read from the trajectory file.
            Default is last time step available
            If timestep is set to all then outputs become lists
        atomlist = list of element strings corresponding to number type
        ratomlist = Boolean for whether or not to return the atom list
        write_file = Boolean for whether or not to write ASE Atoms object to file
    Ouput:
        ASE atoms object containing structure from timestep specified
        if ratomlist: list containing element strings applied to structure corresponding to LAMMPS number
        vlist = list of velocities for each atom in structure at timestep
        flist = list of forcer for each atom in structure at timestep
    """
    f = open(filename,'r')
    if atomlist==False:
        atnum = []
        attype = []
    alist = []
    vlist = []
    flist = []
    timeflag = False
    numberflag = False
    boxflag = False
    atomsflag = False
    for line in f.readlines():
        sp = line.split()
        if timeflag == True:
            timeflag = False
        elif numberflag == True:
            noa = float(sp[0])
            numberflag = False
        elif boxflag == True:
            if boxcount <= 2:
                box.append(float(sp[1]))
                boxcount += 1
            if boxcount > 2:
                boxflag = False
        elif atomsflag == True:
            if na < noa:
                if atomlist:
                    sym = atomlist[[i for i,sym in enumerate(atomlist) if str(i+1)==sp[1]][0]]
                    at = Atom(symbol=sym, position=[float(sp[2]),float(sp[3]),float(sp[4])])
                    a.append(at)
                    v = [float(sp[i]) for i in range(5,8)]
                    vs.append(v)
                    f1 = [float(sp[i]) for i in range(8,11)]
                    fs.append(f1)
                    na += 1
                else:
                    if sp[1] in atnum:
                        sym = attype[[i for i,num in enumerate(atnum) if num==sp[1]][0]]
                        at=Atom(symbol=sym,position=[float(sp[2]),float(sp[3]),float(sp[4])])
                        a.append(at)
                    else:
                        symn = random.choice(range(1,100))
                        at=Atom(symbol=symn,position=[float(sp[2]),float(sp[3]),float(sp[4])])
                        a.append(at)
                        atnum.append(sp[1])
                        attype.append(at.symbol)
                    v = [float(sp[i]) for i in range(5,8)]
                    vs.append(v)
                    f1 = [float(sp[i]) for i in range(8,11)]
                    fs.append(f1)
                    na += 1
            if na >= noa:
                atomsflag = False
                if atomlist==False:
                    atomlist=list(numpy.zeros(len(attype)))
                    for i in range(len(atnum)):
                        atomlist[int(atnum[i])-1]=attype[i]
                alist.append(a)
                vlist.append(vs)
                flist.append(fs)
        elif sp[1] == 'TIMESTEP':
            timeflag = True
        elif sp[1] == 'NUMBER':
            numberflag = True
        elif sp[1] == 'BOX':
            boxflag = True
            box = []
            boxcount = 0
            pbcc = [False,False,False]
            if sp[3] == 'pp':
                pbcc[0] = True
            if sp[4] == 'pp':
                pbcc[1] = True
            if sp[5] == 'pp':
                pbcc[2] = True
        elif sp[1] == 'ATOMS':
            atomsflag = True
            na = 0
            a = Atoms(cell=box,pbc=pbcc)
            vs = []
            fs = []
    f.close()
    if timestep == 'all':
        writefile=False
    if writefile == True:
        write_xyz(filename+'.xyz',alist[timestep],'xyz')
    if timestep == 'all':
        if ratomlist:
            return alist, atomlist, vlist, flist
        else:
            return alist, vlist, flist
    else:
        if ratomlist:
            return alist[timestep], atomlist, vlist[timestep], flist[timestep]
        else:
            return alist[timestep], vlist[timestep], flist[timestep]




import random
import numpy
from MAST.structopt.tools.find_defects import find_defects

def permutation(indiv, Optimizer):
    """Move function to perform Permutation of one atom based on atomlist
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    a1=indiv[0][random.randint(0,indiv[0].get_number_of_atoms()-1)]
    opts=[inds for inds in indiv[0] if inds.symbol !=a1.symbol]
    try:
        a2=opts[random.randint(0,len(opts)-1)]
    except ValueError:
        a2=indiv[0][random.randint(0,indiv[0].get_number_of_atoms()-1)]
        Optimizer.output.write('WARNING: Permutation Mutation attempted on single atom structure system\n')
    indiv[0][a1.index].symbol, indiv[0][a2.index].symbol=a2.symbol, a1.symbol
    Optimizer.output.write('Permutation Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write(repr(a1.symbol) + ' swapped with ' + repr(a2.symbol)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='P'
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


from ase import Atom, Atoms
import numpy
import copy
from ase.calculators.neighborlist import NeighborList
from MAST.structopt.inp_out.write_xyz import write_xyz
from MAST.structopt.tools.calc_dist import calc_dist

def find_defects(solid, bulko, rcutoff, atomlistcheck=False, trackvacs=False, trackswaps=False, debug=False, dcheck = 0.6):
    """Function to find interstitials, vacancies, and substitutional atoms (swaps) in a defected structure.
    Identifies species by comparison to perfect structure.
    Inputs:
        solid = ASE atoms class for defected structure
        bulko = ASE atoms class for perfect structure
        rcutoff = float value of distance to surrounding atoms to include
        atomlistcheck = False/list of atom types and concentrations according to atomlist format
        trackvacs = True/False whether or not to identify vacancies in defect
        trackswaps = True/False whether or not to identify substitutional defects
        debug = False/file object to write debug structures"""
    # Combine perfect and defect structures together
    b = bulko.copy()
    b.extend(solid)
    b.set_pbc(True)
    #Debug: Write solid and bulko to file
    if debug:
        print len(bulko)
        write_xyz(debug,b,'Find Ints: Solid and Bulko')
    # Identify nearest neighbor atoms for each atom in perfect structure
    ntot = len(bulko)
    ctoff1 = [1.2 for one in b]
    nl = NeighborList(ctoff1, bothways=True, self_interaction=False)
    nl.update(b)
    slist = []
    blist = []
    wlist = []
    #Loop over each atom in perfect structure
    for one in range(ntot):
        indices, offsets = nl.get_neighbors(one)
        for index, d in zip(indices,offsets):
            index = int(index)
            if index >= ntot:
                pos = b[index].position + numpy.dot(d,bulko.get_cell())
                natm1 = Atom(position=pos)
                dist, dx, dy, dz = calc_dist(b[one],natm1)
                if dist <= dcheck:
                    #Assume atoms closer than 0.6 Angstroms to be equivalent
                    slist.append(index-ntot)
                    blist.append(one)
                    if b[one].symbol == b[index].symbol:
                        wlist.append(index-ntot)
    #Identify those atoms corresponding to interstitials, vacancies, and substitutions
    oslist = [atm.index for atm in solid if atm.index not in slist]
    vlist = [atm.index for atm in bulko if atm.index not in blist]
    swlist = [atm.index for atm in solid if atm.index not in wlist and atm.index not in oslist]
    # Create Atoms objects for each identified defect
    ntot = len(solid)
    cluster = Atoms()
    for one in oslist:
        cluster.append(solid[one])
    vacant = Atoms()
    if trackvacs==True:
        for one in vlist:
            vacant.append(Atom(symbol=bulko[one].symbol, position=bulko[one].position))
            solid.append(Atom(symbol='X', position=bulko[one].position))
            oslist.append(len(solid)-1)
        stro = 'Cluster Identified with length = {0}\nIdentified {1} vacancies\n'.format(len(cluster),len(vlist))
    swaps = Atoms()
    if trackswaps==True:
        for one in swlist:
            swaps.append(solid[one])
            oslist.append(one)
        stro = 'Cluster Identified with length = {0}\nIdentified {1} swaps\n'.format(len(cluster),len(swlist))
    else:
        stro = 'Cluster Identified with length = {0}\n'.format(len(cluster))
    #Debug: write cluster to file
    if debug:
        b = cluster.copy()
        write_xyz(debug,b,'Find Ints: Cluster')
        debug.flush()
        print 'Found cluster size = ',len(b)
    # Identify atoms surrounding the identified defects in the defected structure
    box=Atoms()
    bulki=Atoms()
    if rcutoff != 0:
        if rcutoff > 2.0:
            cutoffs = [rcutoff for one in solid]
        else:
            cutoffs = [2.0 for one in solid]
        solid.set_pbc(True)
        nl = NeighborList(cutoffs,bothways=True,self_interaction=False)
        nl.update(solid)
        nbatmsd = []
        repinds = []
        for one in oslist:
            if one not in repinds:
                if one < ntot:
                    nbatmsd.append((0,one))
                    repinds.append(one)
            indices, offsets = nl.get_neighbors(one)
            for index,d in zip(indices,offsets):
                index = int(index)
                if index not in repinds and index < ntot:
                    opos = copy.copy(solid[index].position)
                    solid[index].position = solid[index].position + numpy.dot(d,solid.get_cell())
                    dist = solid.get_distance(one,index)
                    solid[index].position = opos
                    if dist <= rcutoff:
                        nbatmsd.append((dist,index))
                        repinds.append(index)
    else:
        nbatmsd = []
        repinds = []
        for one in oslist:
            if one not in repinds:
                if one < ntot:
                    nbatmsd.append((0,one))
                    repinds.append(one)
    nbatmsd=sorted(nbatmsd, key=lambda one:one[0], reverse=True)
    indices=[]
    i=0
    # Select only atoms closest to defects that satisfy concentrations specified by atomlist given in atomlistcheck
    if atomlistcheck:
        natomsbox=sum([c for sym,c,m,u in atomlistcheck])
        if len(nbatmsd) > natomsbox:
            while i<natomsbox:
                a=nbatmsd.pop()
                box.append(solid[a[1]])
                indices.append(a[1])
                i+=1
        elif len(nbatmsd) <= natomsbox:
            for a in nbatmsd:
                box.append(solid[a[1]])
                indices.append(a[1])
                i+=1
            if len(box) < natomsbox:
                try:
                    while True:
                        for n in range(len(nbatmsd)-1,-1,-1):
                            inds, offsets=nl.get_neighbors(nbatmsd[n][1])
                            for one,d in zip(inds,offsets):
                                if len(box) < natomsbox:
                                    if one not in indices and one < ntot:
                                        opos = copy.copy(solid[one].position)
                                        solid[one].position = solid[one].position + numpy.dot(d,solid.get_cell())
                                        dist = solid.get_distance(nbatmsd[n][1],one)
                                        solid[one].position = opos
                                        if dist <= rcutoff*5.0:
                                            box.append(solid[one])
                                            indices.append(one)
                                else:
                                    raise StopIteration()
                            for one,d in zip(inds,offsets):
                                if len(box) < natomsbox:
                                    if one not in indices and one < ntot:
                                        opos = copy.copy(solid[one].position)
                                        solid[one].position = solid[one].position + numpy.dot(d,solid.get_cell())
                                        dist = solid.get_distance(nbatmsd[n][1],one)
                                        solid[one].position = opos
                                        box.append(solid[one])
                                        indices.append(one)
                                else:
                                    raise StopIteration()
                except StopIteration:
                    pass
    # If atomlistcheck is False then use all the atoms in the given cutoff distance
    else:
        for a in nbatmsd:
            box.append(solid[a[1]])
            indices.append(a[1])
    # Add remaining atoms in defect to defected bulk atoms object
    for one in range(len(solid)):
        if one not in indices and one < ntot:
            bulki.append(solid[one])
    #Check for accidental vacancy admissions
    #checklist=[atm for atm in box if atm.symbol=='X']
    #checklist.extend([atm for atm in bulki if atm.symbol=='X'])
    #Set up new individual
    indiv=box.copy()
    bulki.set_cell(bulko.get_cell())
    indiv.set_cell(bulko.get_cell())
    bulki.set_pbc(True)
    indiv.set_pbc(True)
    stro+='New individual ({0} atoms) : {1}\n'.format(len(indiv), indiv)
    stro+='New bulki ({0} atoms) : {1}\n'.format(len(bulki), bulki)
    #Debug: write new indiv to file
    if debug:
        b=indiv.copy()
        write_xyz(debug,b,'Find Ints: New Individual')
        #Debug: write new bulki to file
        b=bulki.copy()
        write_xyz(debug,b,'Find Ints: New Bulki')
        debug.flush()
        print len(bulko)
    
    return indiv,bulki,vacant,swaps,stro


"""Tests for Recipesetup"""

from MAST.recipe.recipesetup import RecipeSetup

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import InputOptions
from MAST.utility import MASTFile
import shutil
testname="recipesetup_test"
testdir = dirutil.get_test_dir(testname)

class TestRecipeSetup(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)
        if not os.path.isdir(os.path.join(testdir,'workdir')):
            os.mkdir('workdir')

    def tearDown(self):
        if os.path.isdir(os.path.join(testdir,'workdir')):
            shutil.rmtree('workdir')

    def test___init__(self):
        rfile=os.path.join(testdir,'personalized')
        wdir=os.path.join(testdir,'workdir')
        struc=None
        iopt=InputOptions()
        myrs=RecipeSetup(recipeFile=rfile, workingDirectory=wdir, inputOptions=iopt, structure=struc)
        self.assertEqual(type(myrs.metafile),MAST.utility.metadata.Metadata)
        #self.testclass.__init__(**kwargs)

    def test_get_my_ingredient_options(self):
        raise SkipTest
        rfile=os.path.join(testdir,'personalized')
        wdir=os.path.join(testdir,'workdir')
        struc=None
        iopt=MAST.parsers.inputparser.InputParser(inputfile='input.inp').parse()
        myrs=RecipeSetup(recipeFile=rfile, workingDirectory=wdir, inputOptions=iopt, structure=struc)
        print iopt
        self.assertEqual(type(myrs.metafile),MAST.utility.metadata.Metadata)
        #self.testclass.get_my_ingredient_options(name, ingredient_type)

    def test_get_method_from_ingredient_type(self):
        raise SkipTest
        #self.testclass.get_method_from_ingredient_type(ingredtype, methodtype="")

    def test_create_recipe_plan(self):
        raise SkipTest
        #self.testclass.create_recipe_plan()

    def test_update_top_meta_for_ingred(self):
        raise SkipTest
        #self.testclass.update_top_meta_for_ingred(myingred)

    def test_create_ingredient(self):
        raise SkipTest
        #self.testclass.create_ingredient(my_ingred_input_options)

    def test_start(self):
        rfile=os.path.join(testdir,'personalized')
        wdir=os.path.join(testdir,'workdir')
        struc=None
        iopt=MAST.parsers.inputparser.InputParser(inputfile='input.inp').parse()
        myrs=RecipeSetup(recipeFile=rfile, workingDirectory=wdir, inputOptions=iopt, structure=struc)
        myrs.start()
        compare_metadata=MASTFile('compare_metadata.txt')
        myrs_metadata=MASTFile(os.path.join(wdir,'metadata.txt'))
        self.assertEqual(myrs_metadata.data, compare_metadata.data)
        dirlist=MAST.utility.dirutil.walkdirs(wdir)
        compare_dirlist=list()
        compare_dirlist.append('perfect_opt1')
        compare_dirlist.append('perfect_opt2')
        compare_dirlist.append('perfect_stat')
        compare_dirlist.append('inducedefect_group1')
        compare_dirlist.append('inducedefect_group3')
        compare_dirlist.append('defect_group1_q=n1_opt1')
        compare_dirlist.append('defect_group1_q=n1_opt2')
        compare_dirlist.append('defect_group1_q=n1_stat')
        compare_dirlist.append('phonon_group1_q=n1_int')
        compare_dirlist.append('phonon_group1_q=n1_int_parse')
        compare_dirlist.append('defect_group3_q=n1_opt1')
        compare_dirlist.append('defect_group3_q=n1_opt2')
        compare_dirlist.append('defect_group3_q=n1_stat')
        compare_dirlist.append('neb_group1-group3_q=n1_opt1')
        compare_dirlist.append('neb_group1-group3_q=n1_opt2')
        compare_dirlist.append('neb_group1-group3_q=n1_stat')
        compare_dirlist.append('phonon_group1-group3_q=n1_host1')
        compare_dirlist.append('phonon_group1-group3_q=n1_host1_parse')
        compare_dirlist_withpath=list()
        for diritem in compare_dirlist:
            compare_dirlist_withpath.append(os.path.join(wdir,diritem))
        mdiff=self.maxDiff
        self.maxDiff=None
        self.assertItemsEqual(dirlist,compare_dirlist_withpath)
        self.maxDiff=mdiff
        #self.testclass.start()




import random
import numpy
from ase import Atom, Atoms

def gen_pop_box(atomlist,size,crystal=False):
    """Function to generate a random structure of atoms.
    Inputs:
        atomlist = List of tuples with structure of atoms and quantity
            [('Sym1',int(concentration1), float(mass1),float(chempotential1)),
            ('Sym2',int(concentration2), float(mass2),float(chempotential2)),...]
        size = Float of length of side of cube within which to generate atoms
        crystal = False/List of crystal cell shape options
            list('cubic','orthorhombic','tetragonal','hexagonal','monoclinic','triclinic')
            cell shape will be adjusted accordingly
    Outputs:
        Returns individual of class Atoms (see ase manual for info on Atoms class)
        and if crystal list provided also outputs combined string with output information
    """
    indiv=Atoms()
    indiv.set_cell([size,size,size])
    # Get list of atom types for all atoms in cluster
    for s,c,m,u in atomlist:
        if c > 0:
            for i in range(c):
                pos = [random.uniform(0,size) for j in range(3)]
                at=Atom(symbol=s,position=pos)
                indiv.append(at)
    if crystal:
        stro=''
        natoms=sum([c for s,c,m,u in atomlist])
        pos=indiv.get_scaled_positions()
        structure=random.choice(crystal)
        cello=indiv.get_cell()
        if structure=='cubic':
            #Set to cubic shape
            an,bn,cn = [numpy.linalg.norm(v) for v in cello]
            a=(an+bn+cn)/3.0
            celln=numpy.array([[a,0,0],[0,a,0],[0,0,a]])
            stro+='Setting cell to cubic\n'
        elif structure=='orthorhombic':
            #Set to orthorhombic
            a=random.uniform(2,natoms**0.3333*size)
            b=random.uniform(2,natoms**0.3333*size)
            c=random.uniform(2,natoms**0.3333*size)
            celln=numpy.array([[a,0,0],[0,b,0],[0,0,c]])
            stro+='Setting cell to orthorhombic\n'
        elif structure=='tetragonal':
            #Set to tetragonal shape
            an,bn,cn = [numpy.linalg.norm(v) for v in cello]
            a=(an+bn)/2.0
            c=cn
            if c==a:
                c=random.uniform(1,natoms**0.3333*size)
            celln=numpy.array([[a,0,0],[0,a,0],[0,0,c]])
            stro+='Setting cell to tetragonal\n'
        elif structure=='hexagonal':
            #Set to hexagonal shape
            an,bn,cn = [numpy.linalg.norm(v) for v in cello]
            a=(an+bn)/2.0
            c=cn
            if c<=a:
                c=random.uniform(a+1,natoms**0.3333*size)
            trans=numpy.array([[1,0,0],[-0.5,(3.0**0.5)/2.0,0],[0,0,1]])
            trans[0]=[a*i for i in trans[0]]
            trans[1]=[a*i for i in trans[1]]
            trans[2]=[c*i for i in trans[2]]
            celln=trans
            stro+='Setting cell to Hexagonal\n'
        elif structure=='monoclinic':
            #Set to monoclinic
            a,b,c = [numpy.linalg.norm(v) for v in cello]
            if a==b:
                b=random.uniform(1,natoms**0.3333*size)
            trans=numpy.array([(1+random.random())*c, 0, (1+random.random())*c])
            celln=numpy.array([[a,0,0],[0,b,0],[0,0,0]])
            celln[2]=trans
            stro+='Setting cell to monoclinic\n'
        elif structure=='triclinic':
            #Set to triclinic
            a,b,c = [numpy.linalg.norm(v) for v in cello]
            celln=numpy.array([[a,0,0],[(1+random.random())*b,(1+random.random())*b,0],[(1+random.random())*c,0,(1+random.random())*c]])
            stro+='Setting cell to triclinic\n'
        indiv.set_cell(celln)
        indiv.set_scaled_positions(pos)
        stro+=repr(indiv.get_cell())+'\n'
        return indiv, stro
    return indiv




from MAST.structopt.inp_out import read_xyz
from MAST.structopt.generate.surface import get_surface_restart_indiv
from MAST.structopt.generate.defect import get_defect_restart_indiv
from MAST.structopt.generate.crystal import get_crystal_restart_indiv
from MAST.structopt.generate.Individual import Individual
import logging

def get_restart_population(Optimizer):
    """
    Function to generate a population from a folder containing existing structures.
    Inputs:
        Optimizer = structopt Optimizer class object
    Outputs:
        pop = List of structopt Individual class objects containing existing structures.
    """
    logger = logging.getLogger(Optimizer.loggername)
    index1 = 0
    Optimizer.output.write('Loading structures from old run\n')
    pop = []
    for i in range(Optimizer.nindiv):
        successflag = False
        try:
            indiv = read_xyz(Optimizer.files[i].name)
            successflag = True
        except IOError,e:
            logger.error('Not enough files in restart to generate population. Resetting nindiv to {0}'.format(i-1),exc_info=True)
            Optimizer.output.write('WARNING: Not enough files in restart to generate population\n')
            Optimizer.nindiv=i-1
            Optimizer.output.write('Resetting nindiv = {0}\n'.format(Optimizer.nindiv))
            Optimizer.output.flush()
            break
        except Exception,e:
            Optimizer.output.write('WARNING: Trouble reading file: {0}'.format(Optimizer.files[i].name),exc_info=True)
            Optimizer.output.write('Error: {0}'.format(e))
            Optimizer.output.flush()
            Optimizer.nindiv-=1
        if successflag:
            Optimizer.output.write('Found good individual = {0}\n'.format(indiv))
            if Optimizer.structure == 'Defect':
                individ = get_defect_restart_indiv(Optimizer,indiv)
            elif Optimizer.structure == 'Surface':
                individ = get_surface_restart_indiv(Optimizer, indiv)
            else:
                indiv.set_cell([Optimizer.size,Optimizer.size,Optimizer.size])
                individ = Individual(indiv)
            individ.index = index1
            if Optimizer.genealogy: 
            	individ.history_index = repr(index1)
            pop.append(individ)
            index1 = index1+1
    if len(pop) == 0:
        raise RuntimeError('Unable to load any structures for Restart')
    # Generate new atomlist concentrations based on cluster+box
    if Optimizer.structure == 'Defect':
        if Optimizer.alloy:
            concents=[]
            for ind in pop:
                cs=[]
                for sym,c,u,m in Optimizer.atomlist:
                    sylen=[atm for atm in ind[0] if atm.symbol==sym]
                    cs.append(len(sylen))
                concents.append(cs)
            natmlist=[0]*len(Optimizer.atomlist)
            for i in range(len(concents[0])):
                alls=[cs[i] for cs in concents]
                avgall=int(sum(alls)/len(alls))
                if avgall>=Optimizer.atomlist[i][1]:
                    natmlist[i]=(Optimizer.atomlist[i][0], avgall,\
                    Optimizer.atomlist[i][2],Optimizer.atomlist[i][3])
                else:
                    natmlist[i]=(Optimizer.atomlist[i][0], Optimizer.atomlist[i][1],\
                    Optimizer.atomlist[i][2],Optimizer.atomlist[i][3])
        else:
            natmlist=[0]*len(Optimizer.atomlist)
            for i in range(len(Optimizer.atomlist)):
                atms1=[inds for inds in pop[0][0] if inds.symbol==Optimizer.atomlist[i][0]]
                natmlist[i]=(Optimizer.atomlist[i][0], len(atms1),Optimizer.atomlist[i][2],\
                Optimizer.atomlist[i][3])
        Optimizer.atomlist=natmlist
        Optimizer.output.write('\n\nNew atomlist concentrations based on cluster+box = {0}\n'.format(
            Optimizer.atomlist))
    return pop




#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Wei Xie
# Last updated: 2014-05-14
##############################################################
import sys
import getopt 
import os
import shutil
import errno
import warnings
import numpy as np
from scipy import stats

import pymatgen as mg
from pymatgen.analysis import ewald
from pymatgen.io import vaspio

import EneVsVm 
import uniformPick    

class writer:
    """
    This is a helper class that copies all console ouput to a text file as well
    """
    def __init__(self, *writers) :
        self.writers = writers

    def write(self, text) :
        for w in self.writers:
            w.write(text)

def mkdir_p(path):
    """
    This is a helper method that makes a directory minicking shell's mkdir -p
    """
    try:
        os.makedirs(path)
    except OSError as exc: 
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise      
def num_atoms_speciewise(inputstruct):
    """
    This is a helper method that output a list giving the number of atoms for 
    each atomic specie in the same order as POTCAR. 
    This method is used instead of pymatgen's composition class because the order 
    from pymatgen's composition class is not necessarify in the same order as POTCAR 
    """

    transitionSiteIndex=[0]
    SpecieLastSite=inputstruct.species[0]
    for i, specieThisSite in enumerate(inputstruct.species):
        if specieThisSite != SpecieLastSite:
            transitionSiteIndex.append(i)
        SpecieLastSite=specieThisSite
    numSpecieTypes=len(transitionSiteIndex)
    numAtomsSpeciewise=[None]*numSpecieTypes
    for j in range(numSpecieTypes-1):
        numAtomsSpeciewise[j]=transitionSiteIndex[j+1]-transitionSiteIndex[j]
    numAtomsSpeciewise[numSpecieTypes-1]=(int(inputstruct.composition.num_atoms)-
                                        transitionSiteIndex[numSpecieTypes-1])

    return numAtomsSpeciewise           
    
def defChg(inputStruct,inputPotcar,inputIncar):
    """
    This is a helper method that calculates the defect charge based on input files
    POSCAR, INCAR and POTCAR (not the file name, but objects of pymatgen's 
    corresponding Poscar, Incar, and Potcar class.
    """
    if ('NELECT' not in inputIncar): 
        raise RuntimeError("cannot find NELECT in input INCAR")
    natoms_el=num_atoms_speciewise(inputStruct)
    zval_el=[None]*len(inputPotcar)
    defchg=0
    for i in range(len(inputPotcar)):
        zval_el[i]=inputPotcar[i].ZVAL
        defchg=defchg+zval_el[i]*natoms_el[i]
    defchg=defchg-inputIncar['NELECT']
    return defchg   
                        
def minVertDist(inputlat):
    """
    This is a helper method that calculates the the minimum distance between the
    vertices of an input 3-D lattice. Is is essentially the shortest distance 
    possible between two points in the space asuuming 3D periodic boundary conditions.
    Used to calculate the shortest defect-defect
    distance in a defected supercell containing one point defect. 
    """
    dummyMat = inputlat.matrix
    return min(np.linalg.norm(dummyMat[0]),
               np.linalg.norm(dummyMat[1]),
               np.linalg.norm(dummyMat[2]),
               np.linalg.norm(dummyMat[0]+dummyMat[1]),
               np.linalg.norm(dummyMat[0]+dummyMat[2]),
               np.linalg.norm(dummyMat[1]+dummyMat[2]),
               np.linalg.norm(dummyMat[0]-dummyMat[1]),
               np.linalg.norm(dummyMat[0]-dummyMat[2]),
               np.linalg.norm(dummyMat[1]-dummyMat[2]),
               np.linalg.norm(dummyMat[0]+dummyMat[1]+dummyMat[2]),
               np.linalg.norm(dummyMat[0]+dummyMat[1]-dummyMat[2]),
               np.linalg.norm(dummyMat[0]-dummyMat[1]+dummyMat[2]),
               np.linalg.norm(dummyMat[0]-dummyMat[1]-dummyMat[2])
               )

def roof_mean(intA,intB):
    """
    This is a helper method that calculates the integer mean of two integers 
    rounded to roof if the direct mean of the two integers are not a integer.
    """
    if (intA+intB)%2 == 0:
        return (intA+intB)/2
    else:
        return (intA+intB+1)/2

#def genLMNs(primordial_struct,minDefDist=5,maxNumAtoms=600,minNumAtoms=64,numStructAsked=5):
def genLMNs(primordial_struct,minDefDist=5,maxNumAtoms=600,numStructAsked=5):
    """
    Calculates a list of LxMxN scaling factors for a input structure given inputs 
    of minmum defect-defect distance, maximum number of atoms that the scaled 
    supercell may have, and how many LxMxN scaling factors to outpout
    """   
    maxLMN=int((maxNumAtoms/primordial_struct.composition.num_atoms)**(1/3.0))
    print ("Generating candidate supercells with the scaling factors"+
           " L/M/N less than or equal to "+str(maxLMN)+".")
    print ("If you want to explore larger scaling factors L/M/N, "+ 
            "please rerun with larger maxNumAtoms input parameter.")
    print (" ")

    #####The following section generates a list of candidates LMN's######   
    Vm_LMN_dict={}
    print ("The following are candiate supercells:")   
    print ("ScalingLMN  " " V_M")
    print ("-----------------")
    for L in range(1,maxLMN+1):
            for M in range(1,maxLMN+1):
                    for N in range(1,maxLMN+1):
                        dummy_sc=primordial_struct.copy()
                        dummy_sc.make_supercell([L,M,N])
                        if ((minVertDist(dummy_sc.lattice) >= minDefDist) and 
                            #(dummy_sc.composition.num_atoms >= minNumAtoms) and
                            (dummy_sc.composition.num_atoms <= maxNumAtoms)):
                            dummy_Vm=np.asscalar(EneVsVm.CalcV_M(dummy_sc))

                            alreadyExist=False
                            for vmraw in Vm_LMN_dict.keys():
                                if abs(vmraw-dummy_Vm)<0.01:
                                    alreadyExist=True
                                    break                                                                                     
                            if not alreadyExist:
                                Vm_LMN_dict[dummy_Vm]=[L,M,N]
                                #print(str([L,M,N]) +"   "+ str(round(dummy_Vm,2)))
                                print(str([L,M,N]) +"   %4.2f" % dummy_Vm)
    print""   
    
    #####The following section selects "Good" LMN's from the candidates list######
    LMN_list_raw=Vm_LMN_dict.values()
    Vm_list_raw=Vm_LMN_dict.keys()
    
    if len(LMN_list_raw) < numStructAsked:
        LMN_list=LMN_list_raw
        warnings.warn("You asked for "+str(numStructAsked)+ 
               " supercells, but totally only "
               +str(len(LMN_list_raw))+
               " supercells were found to fullfile the input requirements!")
    elif len(LMN_list_raw) == numStructAsked:
        LMN_list=LMN_list_raw
    elif type(numStructAsked) != int or numStructAsked <=0:
        raise RuntimeError("You asked for "+str(numStructAsked)+ " supercells, \
            but this input parameter should be a positive integer number.")
    elif numStructAsked == 1:
        VmSelected=sorted(Vm_list_raw)[roof_mean(0,(len(Vm_list_raw)-1))]
        LMN_list=[Vm_LMN_dict[VmSelected]]
    elif numStructAsked == 2:
        VmSelected=[min(Vm_list_raw),max(Vm_list_raw)]
        LMN_list=[Vm_LMN_dict[VmSelected[0]],Vm_LMN_dict[VmSelected[1]]]                             
    else:
        VmSelected=uniformPick.pickSubList(Vm_list_raw,numStructAsked)        
        LMN_list=[]                
        for Vm in VmSelected:
            LMN_list.append(Vm_LMN_dict[Vm])

    print ("The following are selected supercells:")   
    print ("ScalingLMN  " " V_M")
    print ("-----------------")           
    for j in range(len(LMN_list)):
        dummystruct=primordial_struct.copy()
        dummystruct.make_supercell(LMN_list[j])
        print (str(LMN_list[j])+"   %4.2f" % EneVsVm.CalcV_M(dummystruct)) 
    print ""
    return LMN_list

def gensc(LMN_list,perf_primordial_struct, 
          def_primordial_struct,def_primordial_kpnt,
          def_primordial_potcar,def_primordial_incar):
    """
    Generate supercells given input LxMxN scaling factor list, the defected 
    primordial cell's POSCAR, INCAR, POTCAR and KPOINTS and undefected primordial
    cell's CONTCAR's corresponding object generated by pymatgen.
    """

    if ('MAGMOM' in def_primordial_incar):
        print ("WARNING: found MAGMOM in the primordial cell INCAR" 
                "please manually reset MAGMOM in the supercell INCAR "
                "to match it to the number of atoms. Otherwise the job will crash.")
    
    defchg=defChg(def_primordial_struct,def_primordial_potcar,def_primordial_incar)
    print("The defect charge is "+str(defchg)+" in the defected primordial cell.")
    print""               
       
    cwdir = os.getcwd()
    print("To run the following supercells:")
#    print ("ScalingLMN " "  V_M    " " Kpoint  " " Natoms")
    print ("ScalingLMN " " Kpoint  "   "   Label")
    print ("---------------------------------") 
    for j in range(len(LMN_list)):

        dirname=str(LMN_list[j][0])+"x"+str(LMN_list[j][1])+"x"+str(LMN_list[j][2])
        mkdir_p(dirname)
        os.chdir(dirname)
        
        dummystruct=perf_primordial_struct.copy()
        dummystruct.make_supercell(LMN_list[j])
        ####The next section does embeding#######            

        ####The above section does embeding#######
        mg.write_structure(dummystruct,'POSCAR') 
       
        kppra=round((def_primordial_kpnt.kpts[0][0]*
                def_primordial_kpnt.kpts[0][1]*
                def_primordial_kpnt.kpts[0][2])/
                (1/def_primordial_struct.composition.num_atoms))        
        dummykpnt=mg.io.vaspio.Kpoints.automatic_density(dummystruct,kppra)
        dummykpnt.style=def_primordial_kpnt.style
        dummykpnt.write_file('KPOINTS')
                
        dummystruct_natoms_el=num_atoms_speciewise(dummystruct)
        dummynelect=-defchg
        for i in range(len(def_primordial_potcar)):
            dummyZval=def_primordial_potcar[i].ZVAL
            dummynelect=dummynelect+dummyZval*dummystruct_natoms_el[i]
        def_primordial_incar.__setitem__('NELECT',dummynelect)
        def_primordial_incar.write_file('INCAR')
        
        def_primordial_potcar.write_file('POTCAR')
        print (str(LMN_list[j])+"   " \
               #    +"%4.2f"      %EneVsVm.CalcV_M(dummystruct)+"   "+
              +str(dummykpnt.kpts[0][0])+"x"+ \
               str(dummykpnt.kpts[0][1])+"x"+ \
               str(dummykpnt.kpts[0][2])+"   "
               #+str(int(dummystruct.composition.num_atoms))
               +"label="
              +str(LMN_list[j][0])+"x"+ \
               str(LMN_list[j][1])+"x"+ \
               str(LMN_list[j][2])               
               )
 
        os.chdir(cwdir)
        
if __name__ == "__main__":
    saved = sys.stdout
    fout = file('out.log', 'w')
    sys.stdout = writer(sys.stdout, fout)
    
    perfDir = "primordial_perfect"
    defDir = "primordial_defected"
    
    ###read CONTCAR from primordial_perfect directory########
    cwdir=os.getcwd()
    os.chdir(perfDir)      
    perfectContcar=mg.io.vaspio.Poscar.from_file("CONTCAR").structure
    os.chdir(cwdir)

    #read CONTCAR, KPOINTS, POTCAR and INCAR from primordial_defect directory##
    os.chdir(defDir)       
    defectedContcar=mg.io.vaspio.Poscar.from_file("CONTCAR").structure
    defectedKpoints=mg.io.vaspio.Kpoints.from_file('KPOINTS')
    defectedPotcar=mg.io.vaspio.Potcar.from_file('POTCAR')
    defectedIncar=mg.io.vaspio.Incar.from_file('INCAR')
    os.chdir(cwdir)

    ##calculate and select a list of LMN scaling factors to run
    #os.chdir('..')
    LMN_list=genLMNs(perfectContcar,3,600,5)

    ##generate the input files for the above scaling factors
    gensc(LMN_list,perfectContcar,
          defectedContcar,defectedKpoints,defectedPotcar,defectedIncar)

    sys.stdout = saved
    fout.close()


def position_average(atoms, mode='median', trimp = 0.05):
    """Function to calculate the trimmed mean or median
    of the positions of atoms
    Input:
        atoms = ASE Atoms object
        mode = method to use either median or trimmedmean
            default value is median
        trimp = percentage of high and lows to leave out
            default value is 5% == 0.05
    Output:
        [x,y,z] = position of atoms center
    """
    positions = atoms.get_positions()
    xs = []
    ys = []
    zs = []
    for x,y,z in positions:
        xs.append(x)
        ys.append(y)
        zs.append(z)
    xs.sort()
    ys.sort()
    zs.sort()
    if mode=='median':
        xc = xs[len(xs)/2]
        yc = ys[len(ys)/2]
        zc = zs[len(zs)/2]
    else:
        ndel = int(round(trimp*len(xs)))
        xc = sum(xs[ndel:len(xs)-ndel])/(len(xs)-2*ndel)
        yc = sum(ys[ndel:len(ys)-ndel])/(len(ys)-2*ndel)
        zc = sum(zs[ndel:len(zs)-ndel])/(len(zs)-2*ndel)
    return [xc,yc,zc]


import math

def convert_time(t):
    """Function to convert time in seconds to readable string"""
    tdays = math.floor(t/(3600*24.0))
    tleft = t-tdays*3600*24
    thours = math.floor(tleft/3600)
    tleft -=thours*3600
    tmins = math.floor(tleft/60)
    tleft -=tmins*60
    tseconds = tleft
    str = "%02d:%02d:%02d:%02.4f" % (tdays, thours, tmins, tseconds)
    return str




from MAST.structopt.inp_out.read_xyz import read_xyz
from MAST.structopt.generate.Individual import Individual
from ase import Atom, Atoms

def read_individual(indivfile, n=-1):
    """Function to write the data of an individual class object to a flat file
    Input:
        indivfile = String or fileobject for file to be read from
        n = which individual from file to return. Default is last individual written.
            optional All
    Output:
        returns an individual class object or list of individual class objects depending on value of n
    """
    if isinstance(indivfile, str):
        indivfile=open(indivfile, 'r')
    all_lines = indivfile.readlines()
    indivfile.close()
    linen = 0
    all_indivs = []
    while linen < len(all_lines):
        if '----------' in all_lines[linen]:
            individ = Individual(Atoms())
        elif 'Structure information' in all_lines[linen]:
            natomstruct = int(all_lines[linen+1])
            atomstruct = Atoms()
            for i in range(natomstruct):
                a = all_lines[linen+i+3].split()
                sym = a[0]
                position = [float(a[1]),float(a[2]),float(a[3])]
                atomstruct.append(Atom(symbol=sym,position=position))
            individ = Individual(atomstruct)
            linen += 2+natomstruct
        elif 'structure cell' in all_lines[linen]:
            cell_line = all_lines[linen].split('=')
            structcell = eval(cell_line[1])
            individ[0].set_cell(structcell)
        elif 'fitness' in all_lines[linen]:
            fitline = all_lines[linen].split('=')
            individ.fitness = float(fitline[1])
        elif 'history_index' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.history_index = line[1].strip()
        elif 'index' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.index = float(line[1])
        elif 'tenergymx' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.tenergymx = float(line[1])
        elif 'tenergymin' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.tenergymin = float(line[1])
        elif 'energy' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.energy = float(line[1])
        elif 'pressure' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.pressure = float(line[1])
        elif 'volume' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.volume = float(line[1])
        elif 'force' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.force = float(line[1])
        elif 'purebulkenpa' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.purebulkenpa = float(line[1])
        elif 'natomsbulk' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.natomsbulk = float(line[1])
        elif 'fingerprint' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.fingerprint = eval(line[1])
        elif 'swpalist' in all_lines[linen]:
            line = all_lines[linen].split('=')
            individ.swaplist = eval(line[1])
        elif 'bulki cell' in all_lines[linen]:
            line = all_lines[linen].split('=')
            cell = eval(line[1])
            individ.bulki.set_cell(cell)
        elif 'bulki' in all_lines[linen]:
            natoms = int(all_lines[linen+1])
            atomstruct = Atoms()
            for i in range(natoms):
                a = all_lines[linen+i+3].split()
                sym = a[0]
                position = [float(a[1]),float(a[2]),float(a[3])]
                atomstruct.append(Atom(symbol=sym,position=position))
            individ.bulki = atomstruct.copy()
            linen += 2+natoms
        elif 'bulko cell' in all_lines[linen]:
            line = all_lines[linen].split('=')
            cell = eval(line[1])
            individ.bulko.set_cell(cell)
        elif 'bulko' in all_lines[linen]:
            natoms = int(all_lines[linen+1])
            atomstruct = Atoms()
            for i in range(natoms):
                a = all_lines[linen+i+3].split()
                sym = a[0]
                position = [float(a[1]),float(a[2]),float(a[3])]
                atomstruct.append(Atom(symbol=sym,position=position))
            individ.bulko = atomstruct.copy()
            linen += 2+natoms
        elif 'box cell' in all_lines[linen]:
            line = all_lines[linen].split('=')
            cell = eval(line[1])
            individ.box.set_cell(cell)
        elif 'box' in all_lines[linen]:
            natoms = int(all_lines[linen+1])
            atomstruct = Atoms()
            for i in range(natoms):
                a = all_lines[linen+i+3].split()
                sym = a[0]
                position = [float(a[1]),float(a[2]),float(a[3])]
                atomstruct.append(Atom(symbol=sym,position=position))
            individ.box = atomstruct.copy()
            linen += 2+natoms
        elif 'vacancies cell' in all_lines[linen]:
            line = all_lines[linen].split('=')
            cell = eval(line[1])
            individ.vacancies.set_cell(cell)
        elif 'vacancies' in all_lines[linen]:
            natoms = int(all_lines[linen+1])
            atomstruct = Atoms()
            for i in range(natoms):
                a = all_lines[linen+i+3].split()
                sym = a[0]
                position = [float(a[1]),float(a[2]),float(a[3])]
                atomstruct.append(Atom(symbol=sym,position=position))
            individ.vacancies = atomstruct.copy()
            linen += 2+natoms
        elif 'swaps cell' in all_lines[linen]:
            line = all_lines[linen].split('=')
            cell = eval(line[1])
            individ.swaps.set_cell(cell)
        elif 'swaps' in all_lines[linen]:
            natoms = int(all_lines[linen+1])
            atomstruct = Atoms()
            for i in range(natoms):
                a = all_lines[linen+i+3].split()
                sym = a[0]
                position = [float(a[1]),float(a[2]),float(a[3])]
                atomstruct.append(Atom(symbol=sym,position=position))
            individ.swaps = atomstruct.copy()
            linen += 2+natoms
        elif 'Finish' in all_lines[linen]:
            all_indivs.append(individ.duplicate())
        linen+=1
    if n=='All':
        return all_indivs
    else:
        return all_indivs[n]
    
    


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import sys
from MAST.utility import Metadata
from MAST.utility import fileutil
from MAST.utility import MASTFile
from MAST.ingredients.checker import VaspChecker
from MAST.ingredients.checker import VaspNEBChecker
def main(ingname=""):
    """Get the last lattice from an ingredient.
        Args:
            ingname <str>: Ingredient name, full path
        Returns:
            <string>: "Last lattice (Angstroms);scale, a, b, c", ex:
                "Last lattice (Angstroms); scale 1, 4.01 0.2 0, 0 4.01 0, 
                        0.1 0.1 4.01"
    """
    trymeta = "%s/metadata.txt" % ingname
    frontstr = "Last lattice (Angstroms);"
    if os.path.isfile(trymeta):
        mymeta = Metadata(metafile=trymeta)
        myprogram = mymeta.read_data("program")
    else:
        myprogram = "None"
    if myprogram in ['vasp','vasp_neb']:
        if os.path.isdir("%s/01" % ingname):
            tryfile = "%s/01/CONTCAR" % ingname
        else:
            tryfile = "%s/CONTCAR" % ingname
        cfile = MASTFile(tryfile)
        if len(cfile.data) == 0:
            return "%s N/A" % frontstr
        scale = cfile.data[1].strip()
        avec = cfile.data[2].strip()
        bvec = cfile.data[3].strip()
        cvec = cfile.data[4].strip()
        return "%s scale %s, %s, %s, %s" % (frontstr, scale, avec, bvec, cvec)
    else:
        return "%s N/A" % frontstr




def rattle(indiv, Optimizer):
    """Move function to perform rotation of a group of atoms
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
        debug = True/False Boolean
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    indiv[0].rattle(stdev=0.3)
    Optimizer.output.write('Rattle Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='Rat'
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


"""Tests for Basechecker"""

from MAST.ingredients.checker.basechecker import BaseChecker

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import MASTError
from MAST.utility import MASTFile
import shutil
import subprocess
testname="checker_test_base"
print "************************CURDIR:", os.getcwd()
testdir = dirutil.get_test_dir(testname)

class TestBasechecker(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)
        if not os.path.isdir("childdir"):
            os.mkdir("childdir")

    def tearDown(self):
        if os.path.isdir("childdir"):
            shutil.rmtree("childdir")

    def test___init__(self):
        raise SkipTest
        #self.testclass.__init__(allowed_keys, **kwargs)

    def test_is_complete(self):
        raise SkipTest
        #self.testclass.is_complete()

    def test_is_started(self):
        raise SkipTest
        #self.testclass.is_started()

    def test_is_ready_to_run(self):
        raise SkipTest
        #self.testclass.is_ready_to_run()

    def test_get_coordinates_only_structure_from_input(self):
        raise SkipTest
        #self.testclass.get_coordinates_only_structure_from_input()

    def test_softlink_a_file(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        mybc=BaseChecker(allowed,name='files')
        mybc.softlink_a_file('childdir','alphatest')
        myfiles=dirutil.walkfiles("childdir")
        print myfiles
        #self.assertTrue("childdir/CHGCAR" in myfiles)
        listme=subprocess.Popen("ls -l childdir",stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        listres=listme.communicate()[0]
        listme.wait()
        print listres
        self.assertTrue("files/alphatest" in listres)
        #self.testclass.softlink_a_file(childpath, filename)
    def test_softlink_a_file_emptyparentfile(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        mybc=BaseChecker(allowed,name='files')
        mybc.softlink_a_file('childdir','emptyfile')
        myfiles=dirutil.walkfiles("childdir")
        print myfiles
        #self.assertTrue("childdir/CHGCAR" in myfiles)
        listme=subprocess.Popen("ls -l childdir",stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        listres=listme.communicate()[0]
        listme.wait()
        print listres
        self.assertTrue("files/emptyfile" in listres)
    def test_softlink_a_file_no_parent_file(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        mybc=BaseChecker(allowed,name='files')
        self.assertRaises(MASTError,mybc.softlink_a_file,'childdir','nofile')
    def test_softlink_a_file_child_file_exists(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        shutil.copy('files/another_alphatest','childdir/alphatest')
        mybc=BaseChecker(allowed,name='files')
        mybc.softlink_a_file('childdir','alphatest')
        myfiles=dirutil.walkfiles("childdir")
        print myfiles
        #self.assertTrue("childdir/CHGCAR" in myfiles)
        listme=subprocess.Popen("ls -l childdir",stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        listres=listme.communicate()[0]
        listme.wait()
        print listres
        self.assertFalse("files/alphatest" in listres)
        self.assertTrue("childdir/alphatest" in myfiles)
        #self.testclass.softlink_a_file(childpath, filename)
    def test_softlink_a_file_child_softlink_exists(self):
        print "CURDIR: ", os.getcwd()
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        os.chdir("childdir")
        linkme=subprocess.Popen("ln -s ../files/another_alphatest alphatest",shell=True)
        linkme.wait()
        os.chdir(testdir)
        mybc=BaseChecker(allowed,name='files')
        mybc.softlink_a_file('childdir','alphatest')
        myfiles=dirutil.walkfiles("childdir")
        print myfiles
        #self.assertTrue("childdir/CHGCAR" in myfiles)
        listme=subprocess.Popen("ls -l childdir",stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        listres=listme.communicate()[0]
        listme.wait()
        print listres
        self.assertTrue("files/another_alphatest" in listres)
        self.assertTrue("childdir/alphatest" in myfiles)
        #self.testclass.softlink_a_file(childpath, filename)
    def test_copy_a_file(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        mybc=BaseChecker(allowed,name='files')
        mybc.copy_a_file('childdir','alphatest','childalpha')
        self.assertTrue(os.path.isfile('childdir/childalpha'))
    def test_copy_a_file_emptyparentfile(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        mybc=BaseChecker(allowed,name='files')
        mybc.copy_a_file('childdir','emptyfile','childalpha')
        self.assertTrue(os.path.isfile('childdir/childalpha'))
    def test_copy_a_file_no_parent_file(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        mybc=BaseChecker(allowed,name='files')
        self.assertRaises(MASTError,mybc.copy_a_file,'childdir','nofile','childalpha')
    def test_copy_a_file_child_file_exists(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        shutil.copy('files/another_alphatest','childdir/alphatest')
        mybc=BaseChecker(allowed,name='files')
        mybc.copy_a_file('childdir','third_alphatest','alphatest')
        compare=MASTFile("files/another_alphatest")
        myfile=MASTFile("childdir/alphatest")
        self.assertEqual(compare.data, myfile.data)
    def test_copy_a_file_child_softlink_exists(self):
        allowed=dict()
        allowed['name']=(str,"","Directory name")
        os.chdir("childdir")
        linkme=subprocess.Popen("ln -s ../files/another_alphatest alphatest",shell=True)
        linkme.wait()
        os.chdir(testdir)
        mybc=BaseChecker(allowed,name='files')
        mybc.copy_a_file('childdir','third_alphatest','alphatest')
        myfiles=dirutil.walkfiles("childdir")
        print myfiles
        #self.assertTrue("childdir/CHGCAR" in myfiles)
        listme=subprocess.Popen("ls -l childdir",stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        listres=listme.communicate()[0]
        listme.wait()
        print listres
        self.assertTrue("files/another_alphatest" in listres)
        self.assertTrue("childdir/alphatest" in myfiles)
        #self.testclass.softlink_a_file(childpath, filename)



import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.tools.find_defects import find_defects

def permutation_crystal(indiv, Optimizer):
    """Move function to perform Permutation of one atom based on atomlist with entire crystal in Defect
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if Optimizer.structure != 'Defect':
        Optimizer.output.write('WARNING: Crystal extended permutation attempted on non-Defect system. Skipping Mutation\n')
    else:
        indexbulk=len(indiv[0])
        solid=Atoms()
        solid.extend(indiv[0])
        solid.extend(indiv.bulki)
        solid.set_pbc(True)
        solid.set_cell(indiv.bulko.get_cell())
        a1=solid[random.randint(0,indiv[0].get_number_of_atoms()-1)]
        opts=[inds for inds in solid if inds.symbol != a1.symbol]
        try:
            a2=opts[random.randint(0,len(opts)-1)]
        except ValueError:
            a2=indiv[0][random.randint(0,indiv[0].get_number_of_atoms()-1)]
            Optimizer.output.write('WARNING: Permutation Mutation attempted on single atom structure system\n')
        solid[a1.index].symbol, solid[a2.index].symbol=a2.symbol, a1.symbol
        indiv.bulki=solid[indexbulk::]
        indiv[0]=solid[0:indexbulk]
        Optimizer.output.write('Crystal-wide Permutation Mutation performed on individual\n')
        Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
        Optimizer.output.write(repr(a1.symbol) + ' swapped with ' + repr(a2.symbol)+'\n')
        Optimizer.output.write(repr(indiv[0])+'\n')
        for sym,c,m,u in Optimizer.atomlist:
            nc=len([atm for atm in solid if atm.symbol==sym])
            Optimizer.output.write('Defect configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n')
        muttype='PC'
        if indiv.energy==0:
            indiv.history_index=indiv.history_index+'m'+muttype
        else:
            indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
from custodian.vasp import handlers
from pymatgen.core.structure import Structure
import inspect
import os
import logging
from MAST.ingredients.errorhandler import BaseError
class GenericError(BaseError):
    """Generic error-handling functions 
    """
    def __init__(self, **kwargs):
        allowed_keys = {
            'name' : (str, str(), 'Name of directory'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }
        BaseError.__init__(self, allowed_keys, **kwargs)



"""Tests for Chopingredient"""

from MAST.ingredients.chopingredient import ChopIngredient

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import MASTFile
import shutil
testname="chop_test_write"
testdir = dirutil.get_test_dir(testname)

class TestWriteIngredient(unittest.TestCase):
    def setUp(self):
        os.chdir(testdir)
        if not os.path.isdir("writedir"):
            os.mkdir("writedir")
        if not os.path.isdir("writedir/neb_labelinit-labelfin"):
            os.mkdir("writedir/neb_labelinit-labelfin")
        if not os.path.isdir("writedir/single_label1"):
            os.mkdir("writedir/single_label1")
        if not os.path.isdir("writedir/single_phonon_label1"):
            os.mkdir("writedir/single_phonon_label1")

    def tearDown(self):
        for foldername in ["writedir/neb_labelinit-labelfin","writedir/single_label1", "writedir/single_phonon_label1"]:
            try:
                shutil.rmtree(foldername)
            except OSError:
                pass

    def test___init__(self):
        self.assertTrue(True)
        #self.testclass.__init__(**kwargs)

    def test_no_setup(self):
        self.assertTrue(True)
        #self.testclass.no_setup()

    def test_write_neb_from_endpoints_only(self):
        ingdir="writedir/neb_labelinit-labelfin"
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        peinit = MASTFile("files/parent_energy_labelinit")
        peinit.to_file("%s/parent_energy_labelinit" % ingdir)
        pefin = MASTFile("files/parent_energy_labelfin")
        pefin.to_file("%s/parent_energy_labelfin" % ingdir)
        psinit = MASTFile("files/parent_structure_labelinit")
        psinit.to_file("%s/parent_structure_labelinit" % ingdir)
        psfin = MASTFile("files/parent_structure_labelfin")
        psfin.to_file("%s/parent_structure_labelfin" % ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp_neb'
        neblines = list()
        neblines.append(["Cr","0.0 0.9 0.8","0.0 0.8 0.7"])
        neblines.append(["Cr","0.4 0.2 0.1","0.3 0.3 0.2"])
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        kdict['mast_neb_settings']['lines']=neblines
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        mywi.write_neb()
        self.assertTrue(mywi.checker.is_ready_to_run())
        pos_compare_00 = MASTFile("files/POSCAR_00")
        pos_compare_01 = MASTFile("files/POSCAR_01")
        pos_compare_02 = MASTFile("files/POSCAR_02")
        pos_compare_03 = MASTFile("files/POSCAR_03")
        pos_compare_04 = MASTFile("files/POSCAR_04")
        mypos00 = MASTFile("writedir/neb_labelinit-labelfin/00/POSCAR")
        mypos01 = MASTFile("writedir/neb_labelinit-labelfin/01/POSCAR")
        mypos02 = MASTFile("writedir/neb_labelinit-labelfin/02/POSCAR")
        mypos03 = MASTFile("writedir/neb_labelinit-labelfin/03/POSCAR")
        mypos04 = MASTFile("writedir/neb_labelinit-labelfin/04/POSCAR")
        self.assertEqual(pos_compare_00.data, mypos00.data)
        self.assertEqual(pos_compare_01.data, mypos01.data)
        self.assertEqual(pos_compare_02.data, mypos02.data)
        self.assertEqual(pos_compare_03.data, mypos03.data)
        self.assertEqual(pos_compare_04.data, mypos04.data)
        #self.testclass.write_neb()
    def test_write_neb_with_parent_image_structures(self):
        ingdir="writedir/neb_labelinit-labelfin"
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        peinit = MASTFile("files/parent_energy_labelinit")
        peinit.to_file("%s/parent_energy_labelinit" % ingdir)
        pefin = MASTFile("files/parent_energy_labelfin")
        pefin.to_file("%s/parent_energy_labelfin" % ingdir)
        psinit = MASTFile("files/parent_structure_labelinit")
        psinit.to_file("%s/parent_structure_labelinit" % ingdir)
        psfin = MASTFile("files/parent_structure_labelfin")
        psfin.to_file("%s/parent_structure_labelfin" % ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp_neb'
        kdict['mast_coordinates']=list()
        kdict['mast_coordinates'].append('%s/files/POSCAR_coords_01' % testdir)
        kdict['mast_coordinates'].append('%s/files/POSCAR_coords_02' % testdir)
        kdict['mast_coordinates'].append('%s/files/POSCAR_coords_03' % testdir)
        neblines = list()
        neblines.append(["Cr","0.0 0.9 0.8","0.0 0.8 0.7"])
        neblines.append(["Cr","0.4 0.2 0.1","0.3 0.3 0.2"])
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        kdict['mast_neb_settings']['lines']=neblines
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        mywi.write_neb()
        self.assertTrue(mywi.checker.is_ready_to_run())
        pos_compare_00 = MASTFile("files/POSCAR_grafted_00")
        pos_compare_01 = MASTFile("files/POSCAR_grafted_01")
        pos_compare_02 = MASTFile("files/POSCAR_grafted_02")
        pos_compare_03 = MASTFile("files/POSCAR_grafted_03")
        pos_compare_04 = MASTFile("files/POSCAR_grafted_04")
        mypos00 = MASTFile("writedir/neb_labelinit-labelfin/00/POSCAR")
        mypos01 = MASTFile("writedir/neb_labelinit-labelfin/01/POSCAR")
        mypos02 = MASTFile("writedir/neb_labelinit-labelfin/02/POSCAR")
        mypos03 = MASTFile("writedir/neb_labelinit-labelfin/03/POSCAR")
        mypos04 = MASTFile("writedir/neb_labelinit-labelfin/04/POSCAR")
        self.assertEqual(pos_compare_00.data, mypos00.data)
        self.assertEqual(pos_compare_01.data, mypos01.data)
        self.assertEqual(pos_compare_02.data, mypos02.data)
        self.assertEqual(pos_compare_03.data, mypos03.data)
        self.assertEqual(pos_compare_04.data, mypos04.data)

    def test_get_parent_structures(self):
        kdict=dict()
        kdict['mast_program'] = 'vasp_neb'
        neblines = list()
        neblines.append(["Cr","0.0 0.9 0.8","0.0 0.8 0.7"])
        neblines.append(["Cr","0.4 0.2 0.1","0.3 0.3 0.2"])
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        kdict['mast_neb_settings']['lines']=neblines
        ingdir = "writedir/neb_labelinit-labelfin"
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        unsorted_init = MASTFile("unsorted/parent_structure_labelinit")
        #unsorted_init = MASTFile("unsorted/parent_structure_labelinit_scrambled")
        unsorted_init.to_file("%s/parent_structure_labelinit" % ingdir)
        unsorted_fin = MASTFile("unsorted/parent_structure_labelfin")
        unsorted_fin.to_file("%s/parent_structure_labelfin" % ingdir)
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        [sinit, sfin] = mywi.get_parent_structures()
        init_compare = pymatgen.io.vaspio.Poscar.from_file("files/parent_structure_labelinit").structure
        fin_compare = pymatgen.io.vaspio.Poscar.from_file("files/parent_structure_labelfin").structure
        #print sinit
        #print init_compare
        #print sfin
        #print fin_compare
        self.assertEqual(sinit.sites, init_compare.sites)
        self.assertEqual(sinit.lattice, init_compare.lattice)
        self.assertEqual(sfin.sites, fin_compare.sites)
        self.assertEqual(sfin.lattice, fin_compare.lattice)
        #self.testclass.get_parent_structures()

    def test_get_parent_image_structures(self):
        kdict=dict()
        kdict['mast_program'] = 'vasp_neb'
        neblines = list()
        neblines.append(["Cr","0.0 0.9 0.8","0.0 0.8 0.7"])
        neblines.append(["Cr","0.4 0.2 0.1","0.3 0.3 0.2"])
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        kdict['mast_neb_settings']['lines']=neblines
        ingdir = "writedir/neb_labelinit-labelfin"
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        unsorted_01 = MASTFile("unsorted/parent_structure_labelinit-labelfin_01")
        unsorted_01.to_file("%s/parent_structure_labelinit-labelfin_01" % ingdir)
        unsorted_02 = MASTFile("unsorted/parent_structure_labelinit-labelfin_02")
        unsorted_02.to_file("%s/parent_structure_labelinit-labelfin_02" % ingdir)
        unsorted_03 = MASTFile("unsorted/parent_structure_labelinit-labelfin_03")
        unsorted_03.to_file("%s/parent_structure_labelinit-labelfin_03" % ingdir)
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        imstrs = mywi.get_parent_image_structures()
        compare_01 = pymatgen.io.vaspio.Poscar.from_file("files/parent_structure_labelinit-labelfin_01").structure
        compare_02 = pymatgen.io.vaspio.Poscar.from_file("files/parent_structure_labelinit-labelfin_02").structure
        compare_03 = pymatgen.io.vaspio.Poscar.from_file("files/parent_structure_labelinit-labelfin_03").structure
        self.assertEqual(imstrs[0].sites, compare_01.sites)
        self.assertEqual(imstrs[0].lattice, compare_01.lattice)
        self.assertEqual(imstrs[1].sites, compare_02.sites)
        self.assertEqual(imstrs[1].lattice, compare_02.lattice)
        self.assertEqual(imstrs[2].sites, compare_03.sites)
        self.assertEqual(imstrs[2].lattice, compare_03.lattice)
        #self.testclass.get_parent_image_structures()

    def test_place_parent_energy_files(self):
        ingdir="writedir/neb_labelinit-labelfin"
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        peinit = MASTFile("files/parent_energy_labelinit")
        peinit.to_file("%s/parent_energy_labelinit" % ingdir)
        pefin = MASTFile("files/parent_energy_labelfin")
        pefin.to_file("%s/parent_energy_labelfin" % ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp_neb'
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        os.mkdir(ingdir + '/00')
        os.mkdir(ingdir + '/04')
        mywi.place_parent_energy_files()
        self.assertTrue(os.path.isfile(ingdir + '/00/OSZICAR'))
        self.assertTrue(os.path.isfile(ingdir + '/04/OSZICAR'))
        oszinit = MASTFile(ingdir + "/00/OSZICAR")
        oszfinal = MASTFile(ingdir + "/04/OSZICAR")
        self.assertEqual(oszinit.data, peinit.data)
        self.assertEqual(oszfinal.data, pefin.data)
        #self.testclass.place_parent_energy_files()

    def test_write_neb_subfolders(self):
        ingdir="writedir/neb_labelinit-labelfin"
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        parent_01 = MASTFile("statfiles/parent_structure_labelinit-labelfin_01")
        parent_01.to_file(ingdir + "/parent_structure_labelinit-labelfin_01")
        parent_02 = MASTFile("statfiles/parent_structure_labelinit-labelfin_02")
        parent_02.to_file(ingdir + "/parent_structure_labelinit-labelfin_02")
        parent_03 = MASTFile("statfiles/parent_structure_labelinit-labelfin_03")
        parent_03.to_file(ingdir + "/parent_structure_labelinit-labelfin_03")
        parent_init = MASTFile("statfiles/parent_structure_labelinit")
        parent_init.to_file(ingdir + "/parent_structure_labelinit")
        parent_fin = MASTFile("statfiles/parent_structure_labelfin")
        parent_fin.to_file(ingdir + "/parent_structure_labelfin")
        peinit = MASTFile("statfiles/parent_energy_labelinit")
        peinit.to_file("%s/parent_energy_labelinit" % ingdir)
        pefin = MASTFile("statfiles/parent_energy_labelfin")
        pefin.to_file("%s/parent_energy_labelfin" % ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp'
        neblines = list()
        neblines.append(["Cr","0.0 0.9 0.8","0.0 0.8 0.7"])
        neblines.append(["Cr","0.4 0.2 0.1","0.3 0.3 0.2"])
        neblines.append(["Cr","0.29 0.05 0.05","0.01 0.01 0.98"])
        neblines.append(["Ni","0.61 0.99 0.98","0.25 0.01 0.97"])
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        kdict['mast_neb_settings']['lines']=neblines
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        mywi.write_neb_subfolders()
        self.assertFalse(os.path.isfile("%s/00/submit.sh" % ingdir))
        self.assertTrue(os.path.isfile("%s/01/submit.sh" % ingdir))
        self.assertTrue(os.path.isfile("%s/02/submit.sh" % ingdir))
        self.assertTrue(os.path.isfile("%s/03/submit.sh" % ingdir))
        self.assertFalse(os.path.isfile("%s/04/submit.sh" % ingdir))
        oszinit = MASTFile("%s/00/OSZICAR" % ingdir)
        oszfin = MASTFile("%s/04/OSZICAR" % ingdir)
        pos00 = MASTFile("%s/00/POSCAR" % ingdir)
        pos01 = MASTFile("%s/01/POSCAR" % ingdir)
        pos02 = MASTFile("%s/02/POSCAR" % ingdir)
        pos03 = MASTFile("%s/03/POSCAR" % ingdir)
        pos04 = MASTFile("%s/04/POSCAR" % ingdir)
        self.assertEqual(oszinit.data, peinit.data)
        self.assertEqual(oszfin.data, pefin.data)
        self.assertEqual(pos01.data, parent_01.data)
        self.assertEqual(pos02.data, parent_02.data)
        self.assertEqual(pos03.data, parent_03.data)
        self.assertEqual(pos00.data, parent_init.data)
        self.assertEqual(pos04.data, parent_fin.data)


        #self.testclass.write_neb_subfolders()

    def test_write_singlerun(self):
        ingdir="writedir/single_label1"
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp'
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        mywi.write_singlerun()
        self.assertTrue(mywi.checker.is_ready_to_run())
        #self.testclass.write_singlerun()

    def test_write_phonon_multiple(self):
        ingdir="writedir/single_phonon_label1"
        topmetad = MASTFile("files/top_metadata_single_phonon")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single_phonon")
        metad.to_file("%s/metadata.txt" % ingdir)
        shutil.copy("files/CHGCAR", ingdir)
        shutil.copy("files/WAVECAR", ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp'
        kdict['mast_phonon_settings']=dict()
        kdict['mast_phonon_settings']['phonon_center_site']="0.33 0.25 0.0"
        kdict['mast_phonon_settings']['phonon_center_radius']="1"
        kdict['mast_phonon_settings']['threshold']="0.075"
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        mywi.write_phonon_multiple()
        self.assertTrue(os.path.isdir("%s/phon_01" % ingdir))
        self.assertTrue(os.path.isdir("%s/phon_02" % ingdir))
        self.assertTrue(os.path.isdir("%s/phon_03" % ingdir))
        mywi.checker.keywords['name'] = "%s/phon_01" % ingdir
        self.assertTrue(mywi.checker.is_ready_to_run())
        mywi.checker.keywords['name'] = "%s/phon_02" % ingdir
        self.assertTrue(mywi.checker.is_ready_to_run())
        mywi.checker.keywords['name'] = "%s/phon_03" % ingdir
        self.assertTrue(mywi.checker.is_ready_to_run())
        compare_phon01 = MASTFile("files/phon_01_POSCAR")
        compare_phon02 = MASTFile("files/phon_02_POSCAR")
        compare_phon03 = MASTFile("files/phon_03_POSCAR")
        phon01 = MASTFile("%s/phon_01/POSCAR" % ingdir)
        phon02 = MASTFile("%s/phon_02/POSCAR" % ingdir)
        phon03 = MASTFile("%s/phon_03/POSCAR" % ingdir)
        self.assertEqual(phon01.data, compare_phon01.data)
        self.assertEqual(phon02.data, compare_phon02.data)
        self.assertEqual(phon03.data, compare_phon03.data)
        #self.testclass.write_phonon_multiple()

    def test_write_phonon_single(self):
        ingdir="writedir/single_phonon_label1"
        topmetad = MASTFile("files/top_metadata_single_phonon")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single_phonon")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp'
        kdict['mast_phonon_settings']=dict()
        kdict['mast_phonon_settings']['phonon_center_site']="0.33 0.25 0.0"
        kdict['mast_phonon_settings']['phonon_center_radius']="1"
        kdict['mast_phonon_settings']['threshold']="0.075"
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        mywi.write_phonon_single()
        self.assertTrue(mywi.checker.is_ready_to_run())
        compare_phon_single = MASTFile("files/phon_single_POSCAR")
        phon_singlepos = MASTFile("%s/POSCAR" % ingdir)
        self.assertEqual(phon_singlepos.data, compare_phon_single.data)
        #self.testclass.write_phonon_single()

    def test_get_my_phonon_params(self):
        ingdir="writedir/single_phonon_label1"
        topmetad = MASTFile("files/top_metadata_single_phonon")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single_phonon")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp'
        kdict['mast_phonon_settings']=dict()
        kdict['mast_phonon_settings']['phonon_center_site']="0.33 0.25 0.0"
        kdict['mast_phonon_settings']['phonon_center_radius']="1"
        kdict['mast_phonon_settings']['threshold']="0.075"
        my_structure=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        [mysite, myrad, mythresh] = mywi.get_my_phonon_params()
        self.assertEqual(mysite,"0.33 0.25 0.0")
        self.assertEqual(myrad,"1")
        self.assertEqual(mythresh,"0.075")
        #self.testclass.get_my_phonon_params()

    def test_write_singlerun_automesh(self):
        ingdir="writedir/single_label1"
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']='pbe'
        kdict['mast_program']='vasp'
        kdict['mast_kpoint_density']='1000'
        mypos=pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure")
        mypos.write_file("writedir/single_label1/POSCAR")
        mywi = ChopIngredient(name=ingdir,program_keys=kdict,structure=mypos.structure)
        mywi.write_singlerun_automesh()
        mykpts = pymatgen.io.vaspio.Kpoints.from_file("writedir/single_label1/KPOINTS")
        self.assertEqual(mykpts.to_dict['kpoints'][0],[6,6,6])
        #print mykpts
        self.assertTrue(mywi.checker.is_ready_to_run())
        #self.testclass.write_singlerun()



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
from custodian.vasp import handlers
from pymatgen.core.structure import Structure
import logging
import inspect
import os
from MAST.ingredients.errorhandler import BaseError
from MAST.ingredients.errorhandler import masterrorhandlers 
from MAST.utility import MASTFile
#from MAST.utility import loggerutils
class VaspError(BaseError):
    """VASP error-handling functions (wraps custodian)
    """
    def __init__(self, **kwargs):
        allowed_keys = {
            'name' : (str, str(), 'Name of directory'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }
        BaseError.__init__(self, allowed_keys, **kwargs)

    def get_error_handlers(self):
        """Get error handler list from custodian.vasp.handlers.
            This is a dynamic function because the Materials Project is
            continuously adding new handlers.
        """
        handlerdict = BaseError.get_error_handlers(self)
        handlerlist = inspect.getmembers(handlers)
        for htry in handlerlist:
            hname = htry[0]
            if 'ErrorHandler' in hname and not (hname == 'ErrorHandler'):
                handlerdict[hname]=htry[1]
        return handlerdict


    def set_handler_inputs(self):
        """We may want to override the files that are being checked, which are
            usually vasp.out or vasprun.xml.
            Use key as error handler class name (like 'VaspErrorHandler').
            Use mast_skip to skip the error.
        """
        handler_input_d=dict()
        handler_input_d['VaspErrorHandler']=["OUTCAR"]
        handler_input_d['DentetErrorHandler']=["OUTCAR"]
        handler_input_d['UnconvergedErrorHandler']=["vasprun.xml"]
        handler_input_d['PoscarErrorHandler']=["OUTCAR"]
        handler_input_d['TripleProductErrorHandler']=["OUTCAR"]
        #handler_input_d['FrozenJobErrorHandler']="mast_skip"
        #handler_input_d['NonConvergingErrorHandler']="mast_skip"
        #handler_input_d['MeshSymmetryErrorHandler']="mast_skip"
        handler_input_d['MASTWalltimeErrorHandler']=[self.keywords['name'],["OUTCAR","OSZICAR","CONTCAR","POSCAR","XDATCAR"],["CONTCAR"],["POSCAR"]]
        handler_input_d['MASTMemoryErrorHandler']=[self.keywords['name'],self.keywords,["OUTCAR","OSZICAR","CONTCAR","XDATCAR"]]
        handler_input_d['VaspReachedNSWErrorHandler']=[self.keywords['name'],["OUTCAR","OSZICAR","CONTCAR","POSCAR","XDATCAR"],["CONTCAR"],["POSCAR"]]

        return handler_input_d

    def clean_up_directory(self):
        """Clean up directory."""
        return BaseError.clean_up_directory(self, ["OUTCAR","OSZICAR"])



def ofile(name):
    l=open(name,'a')
    return l

def restart_output(Optimizer):
    outputoptions = ['output', 'summary','Genealogyfile', 'tenergyfile', 'debugfile','fpfile', 'fpminfile']
    outfiles = {'files':None, 'ifiles':None, 'output':None, 'summary':None,
        'Genealogyfile':None, 'tenergyfile':None, 'debugfile':None,
        'fpfile':None, 'fpminfile':None}
    for one in outputoptions:
        outpar = eval('Optimizer.{0}'.format(one))
        if outpar:
            outfiles[one] = eval("open({0},'a')".format(repr(outpar)))
    outfiles['files'] = map(ofile,Optimizer.files)
    if Optimizer.indiv_defect_write:
        outfiles['ifiles'] = map(ofile,Optimizer.ifiles)
    return outfiles



from MAST.structopt.tools import eval_energy
from MAST.structopt.fitness import totalenfit
from MAST.structopt.inp_out.write_xyz import write_xyz
import logging
import math
import os

def formationenergy(indiv, Optimizer):
    """Function to calculate formation energy fitness of individual.
    Input:
        indiv = structopt Individual class object to be evaluated
        Optimizer = structopt Optimizer class object
    Output:
        indiv = structopt Individual class object with new fitness.
    """
    if Optimizer.structure != 'Defect':
        if Optimizer.generation <1:
            print 'WARNING: Formation Energy fitness calculator only available for Defect structure simulation.'
            print '     Using total energy fitness calculator instead'
        indiv = totalenfit(indiv,Optimizer)
        stro = ''
    else:
        logger = logging.getLogger(Optimizer.loggername)
        #logger = initialize_logger(Optimizer.loggername)
        starting = indiv.duplicate()
        cwd = os.getcwd()
        try:
            outs = eval_energy(Optimizer,indiv)
        except Exception, e:
            logger.warn('Error in energy evaluation: {0}'.format(e), exc_info=True)
            stro = 'ERROR: Problem in Energy Evaluation'
            print stro
            print e
            stro += '\n' + repr(e)
            os.chdir(cwd)
            f=open('problem-structures.xyz','a')
            totalsol = indiv[0].copy()
            totalsol.extend(indiv.bulki)
            write_xyz(f,totalsol,data='Starting structure hindex={0}'.format(indiv.history_index))
            indiv.energy = 10000
            f.close()
            print '    Writing structure to problemstructures.xyz file. Structure (hindex) : '+indiv.history_index
            print '    Setting individual energy to 50000.'
            outs = [10000, starting.bulki, starting, stro]
        indiv.energy = outs[0]
        stro=outs[3]
        indiv.bulki = outs[1]
        #fit = (indiv.purebulkenpa*(indiv[0].get_number_of_atoms()+indiv.bulki.get_number_of_atoms()) - indiv.energy)/(indiv[0].get_number_of_atoms()+indiv.bulki.get_number_of_atoms()-indiv.natomsbulk)
        solid=Atoms()
        solid.extend(indiv[0])
        solid.extend(indiv.bulki)
        fit=indiv.energy
        passflag = True
        if abs(fit) > Optimizer.energy_cutoff_factor*(len(indiv[0])+len(indiv.bulki)):
            fit=10000
            message = 'Warning: Found oddly large energy from Lammps in structure HI={0}'.format(indiv.history_index)
            logger.warn(message)
            print message
            print '    Setting fitness to 10000'
            passflag=False
        if math.isnan(fit):
            logger.warn('Found NAN energy structure HI={0}'.format(indiv.history_index))
            fit=10000
            passflag=False
            indiv.energy = 10000
        if passflag:
            for sym,c,m,u in Optimizer.atomlist:
                nc=len([atm for atm in solid if atm.symbol==sym])
                fit-= float(nc)*float(u)
        indiv.fitness=fit
    return indiv, stro



from MAST.structopt.tools.lammps import LAMMPS
from ase.calculators.vasp import Vasp

def setup_fixed_region_calculator(Optimizer):
    """Function to set up a fixed region calculator for use with LAMMPS and ASE
    """
    calc = self.calc
    pms = copy.deepcopy(calc.parameters)
    nat = sum([c for sym,c,m,mu in self.atomlist])
    try:
        pms['mass'][len(pms['mass'])-1] += '\ngroup RO id >= ' +repr(nat)+'\nfix freeze RO setforce 0.0 0.0 0.0\n'
    except KeyError:
        pms['pair_coeff'][0] += '\ngroup RO id >= '+repr(nat)+'\nfix freeze RO setforce 0.0 0.0 0.0\n'
    ncalc = LAMMPS(parameters=pms, files=calc.files,
    keep_tmp_files=calc.keep_tmp_files, tmp_dir=calc.tmp_dir)
    return ncalc


import random
from MAST.structopt.tools.find_defects import find_defects

def rotation(indiv, Optimizer):
    """Move function to perform rotation of a group of atoms
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            atms,indb,vacant,swap,stro = find_defects(indiv[0],Optimizer.solidbulk,0)
        else:
            atms = indiv[0]
    else:
        atms=indiv[0]
    nat=len(atms)
    if nat != 0:
        if nat<=1:
            natrot2=1
            natrot1=0
        elif nat<=6:
            natrot2=2
            natrot1=0
        else: 
            natrot1=random.randint(1,nat/2)
            natrot2=random.randint(2,(nat-1)/2)
            if natrot2 >= natrot1:
                natrot2 +=1
            else:
                natrot1, natrot2 = natrot2, natrot1
        natrot=natrot2 - natrot1
        atmsr=atms[natrot1:natrot2]
        del atms[natrot1:natrot2]
        ax=['x', '-x','y','-y','z','-z']
        rax=ax[random.randint(0,len(ax)-1)]
        rang=random.uniform(30,180)
        #rang=random.random()*90
        atmsr.rotate(rax,a=rang,center='COM',rotate_cell=False)
        atms.extend(atmsr)
        if Optimizer.structure=='Defect':
            if Optimizer.isolate_mutation:
                atms.extend(indb)
        indiv[0]=atms.copy()
    else:
        natrot=0
        rax=0
        rang=0
    Optimizer.output.write('Rotation Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write('Number of atoms rotated = '+repr(natrot)+'\n')
    Optimizer.output.write('Rotation vector = '+repr(rax)+'\n')
    Optimizer.output.write('Rotation angle = '+repr(rang)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='R'+repr(natrot)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.tools.find_defects import find_defects

def lattice_alteration_group(indiv, Optimizer):
    """Move function to perform Lattice Alteration of group of atoms based on location
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            indc,indb,vacant,swap,stro = find_defects(indiv[0],Optimizer.solidbulk,0)
            atms=indc.copy()
        else:
            atms=indiv[0].copy()
    else:
        atms=indiv[0].copy()
    if len(atms) != 0:
        try:
            natomsmove=random.randint(1,len(atms)/5)
        except ValueError:
            natomsmove=1
        #Select random position in cluster
        cellx=numpy.maximum.reduce(atms.get_positions())
        cellm=numpy.minimum.reduce(atms.get_positions())
        pt=[random.uniform(cellm[0],cellx[0]),random.uniform(cellm[1],cellx[1]),random.uniform(cellm[2],cellx[2])]
        #Get distance of atoms from random point
        atpt=Atom(position=pt)
        atms.append(atpt)
        dist=[]
        for i in range(len(atms)-1):
            dist.append(atms.get_distance(i,len(atms)-1))
        atms.pop()
        dlist=zip(dist,atms)
        dlist=sorted(dlist, key=lambda one: one[0], reverse=True)
        # Select atoms closest to random point
        atmst=Atoms()
        indexlist=[]
        for i in range(natomsmove):
            atmst.append(dlist[i][1])
            indexlist.append(dlist[i][1].index)
        trans=(random.uniform(0,cellx[0]-cellm[0]), random.uniform(0,cellx[1]-cellm[1]), random.uniform(0,cellx[2]-cellm[2]))
        atmst.translate(trans)
        for i in range(len(indexlist)):
            index=indexlist[i]
            atms[index].position=atmst[i].position
        if Optimizer.structure=='Defect':
            if Optimizer.isolate_mutation:
                indiv[0]=atms.copy()
                indiv[0].extend(indb)
            else:
                indiv[0] = atms.copy()
        else:
            indiv[0]=atms.copy()
    else:
        natomsmove=0
        trans=0
    Optimizer.output.write('Group Lattice Alteration Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write('Number of atoms moved = '+repr(natomsmove)+'\n')
    Optimizer.output.write(repr(trans)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='LAGC'+repr(natomsmove)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


########
ALGO
AMIX
APACO
BMIX
DEPER
DIPOL
EBREAK
EDIFF
EDIFFG
EMAX
EMIN
ENCUT
GGA
I_CONSTRAINED_M
IALGO
IBRION
ICHARG
IMAGES
INIWAV
ISIF
ISMEAR
ISPIN
ISPIND
ISTART
ISYM
IWAVPR
KBLOCK
KPAR
LAMBDA
LASYNC
LCHARG
LCLIMB
LCORR
LDAU
LDAUJ
LDAUL
LDAUPRINT
LDAUTYPE
LDAUU
LELF
LMAXMIX
LNONCOLLINEAR
LORBIT
LREAL
LSCALAPACK
LSCALU
LVHAR
LVTOT
LWAVE
M_CONSTR
MAGMOM
NBANDS
NBLK
NBLOCK
NELECT
NELM
NELMDL
NELMIN
NFREE
NGX
NGXF
NGY
NGYF
NGZ
NGZF
NPACO
NPAR
NSW
NUPDOWN
NWRITE
POMASS
POTIM
PREC
ROPT
RWIGS
SIGMA
SMASS
SPRING
SYMPREC
SYSTEM
TEBEG
TEEND
TIME
VOSKOWN
WEIMIN
ZVAL
########



########
potential_file
structopt_input_file
exec_mast
parallel
filename
loggername
atomlist
structure
natoms
optimizer_type
nindiv
genealogy
output_format
allenergyfile
best_inds_list
number_of_bests
indiv_defect_write
vacancy_output
lattice_concentration
postprocessing
genealogytree
seed
forcing
debug
algorithm_type
migration_intervals
migration_percent
fingerprinting
fpbin
fpcutoff
bulkfp
fixed_region
rattle_atoms
constrain_position
restart_ints
restart
r_ab
size
generate_flag
solidbulk
sf
supercell
solidfile
solidcell
evalsolid
finddefects
trackvacs
trackswaps
random_loc_start
random_vac_start
solidbulk
purebulkenpa
natomsbulk
surfacefile
surftopthick
surfacecell
cell_shape_options
alloy
calc_method
vaspcalc
pair_style
bopcutoff
buckcutoff
buckparameters
ps_name
pair_coeff
ps_other
pot_file
lammps_keep_files
lammps_thermo_steps
lammps_min
lammps_min_style
large_box_size
asemin
ase_min_fmax
ase_min_maxsteps
cxpb
cx_scheme
selection_scheme
mutpb
mutation_options
bh_temp
bh_steps
mutant_add
quench_max_temp
quench_min_temp
quench_n_steps_1
quench_n_steps_2
quench_step_size
isolate_mutation
fitness_scheme
energy_cutoff_factor
stem_coeff
stem_parameters 
stem_keep_files
constrain_swaps
swaplist
natural_selection_scheme
tournsize
fusslimit
metropolis_temp
convergence_scheme
maxgen
reqrep
tolerance
predator
adaptbegin
adaptmultiplier
demin
mark
restart_optimizer
genrep
minfit
generation
convergence
overrideconvergence
Runtimes
Evaluations
CXs
Muts
cxattempts
mutattempts
optimizerfile
########



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Wei Xie
# Last updated: 2014-05-19
##############################################################
import sys, getopt, os
import shutil
import matplotlib.pyplot as plt
import numpy as np
from scipy import stats

import pymatgen as mg
from pymatgen.analysis import ewald
from pymatgen.io import vaspio

import GenSC

class writer :
        def __init__(self, *writers) :
                self.writers = writers

        def write(self, text) :
                for w in self.writers :
                        w.write(text)
                        
def psTotEne(oszicar):
    dummyOszcar = vaspio.vasp_output.Oszicar(oszicar).electronic_steps
    TotEne = dummyOszcar[len(dummyOszcar)-1][len(dummyOszcar[len(dummyOszcar)-1])-1]['E']
    return TotEne
    
def CalcV_M(structure):
    dummyPoscarLatStr = mg.core.structure.Structure(structure.lattice,["F-"],[[0,0,0]])
    V_M = ewald.EwaldSummation(dummyPoscarLatStr).total_energy*(-2)
    return V_M


        
def collectEformV_M(perfectDir,defectDirs,defectChg,mu_e,mu_i):
    #mu_e is the chemical potential of electron 
    #mu_i is a dictionary of chemical potentials for atomic species
    
    Eform = [None]*len(defectDirs)
    V_M = [None]*len(defectDirs)

 
    cwdir = os.getcwd()
    os.chdir(perfectDir)
    
    prf=mg.io.vaspio.vasp_output.Vasprun("vasprun.xml")

    prf_ene=prf.final_energy
    #TotEnePr = psTotEne("OSZICAR")

    prf_strct=prf.structures[len(prf.structures)-1]
    
    prf_comps=prf_strct.composition.element_composition
    #perfect_comps = mg.io.vaspio.Poscar.from_file("CONTCAR").structure.composition    
    prf_natom = prf_comps.num_atoms
    
    prf_elmnts = prf_comps.elements
    
    #gap,cbm,vbm,ifdirectgap=prf.eigenvalue_band_properties
    #print (defectDirs[i]+" "+str(V_M[i])+" "+str(gap)+" "+str(cbm)+" "+str(vbm))

    os.chdir(cwdir)
    
    all_elmnts = prf_elmnts               
    for i in range(len(defectDirs)):
        os.chdir(defectDirs[i])
        
        dfct=mg.io.vaspio.vasp_output.Vasprun("vasprun.xml")

        dfct_strct=dfct.structures[len(dfct.structures)-1]
        #defct_strct=mg.io.vaspio.Poscar.from_file("CONTCAR").structure
        
        V_M[i] = CalcV_M(dfct_strct)

        dfct_comps=dfct_strct.composition.element_composition    
        #defct_comps=defct_strct.composition             

        dfct_natom = dfct_comps.num_atoms
        
        dfct_ene = dfct.final_energy
        #defct_ene = psTotEne("OSZICAR")
        
        scalingF = round(dfct_natom/prf_natom)
                    
        dfct_elmnts = dfct_comps.elements
        #print dfct_elmnts
        
        if i == 0:

            for dummy_elmnt1 in dfct_elmnts:
                if dummy_elmnt1 not in all_elmnts:
                    all_elmnts.append(dummy_elmnt1)

            all_amnt_change=[None]*len(all_elmnts) 
            
            for j, dummy_elmnt2 in enumerate(all_elmnts):
                if dummy_elmnt2 in prf_elmnts:
                    dummy_prf_amnt = prf_comps.__getitem__(dummy_elmnt2)
                else:
                    dummy_prf_amnt = 0
                if dummy_elmnt2 in dfct_elmnts:
                    dummy_dfct_amnt = dfct_comps.__getitem__(dummy_elmnt2)
                else:
                    dummy_dfct_amnt = 0
                
                all_amnt_change[j]=scalingF*dummy_prf_amnt-dummy_dfct_amnt                                    

        Eform[i] =  dfct_ene - scalingF*prf_ene 

        #Eform[i] += defectChg*(vbm+mu_e*gap)
        Eform[i] += defectChg*mu_e

        for k in range(len(all_elmnts)):

            if all_amnt_change[k] != 0:

                if all_elmnts[k].symbol in mu_i:                    
                    Eform[i] += mu_i[all_elmnts[k].symbol]*all_amnt_change[k]
                else:
                    raise RuntimeError("Chemical potential of "+all_elmnts[k].symbol
                                        +"is not provided!")       

        os.chdir(cwdir)

    return(V_M,Eform)

def linearFit(listX,listY):
    listXnp = np.array(listX)
    listYnp = np.array(listY)
    slope, intercept, r_value, p_value, std_err = stats.linregress(listXnp,listYnp)
    
    print "The results of linear fitting:"
    print "slope: ", slope
    print "intercept: ", intercept
    print "correlation coefficent r: ", r_value
    print "standard deviation: ", std_err
    print "" 
    return [slope,intercept,r_value,std_err]   
    
def plotFit(listX,listY,Xlabel,Ylabel,slpIntcpt,figTitle,dirNames):
    listXnp = np.array(listX)
    listYnp = np.array(listY)
    listYnpCorrectted = listYnp-slpIntcpt[0]*listXnp

    Left=min(listX)-0.2
    Right=max(listX)+0.2
        
    listXnpFitted=np.arange(Left,Right,(Right-Left)/6)
    listXnpFitted=np.append(listXnpFitted,Right)
    listYnpFitted = slpIntcpt[0]*listXnpFitted + slpIntcpt[1]
    listYnpCorrecttedFitted = listXnpFitted*0+slpIntcpt[1]
    
    fig = plt.figure()
    ax = fig.add_axes([0.12,0.1,0.8,0.8])
    
    #plt.plot(listXnp,listYnp,'ro',listXnpFitted,listYnpFitted,'r-',
    #         listXnp,listYnpCorrectted,'bo',listXnpFitted,listYnpCorrecttedFitted,'b--')
    line1=plt.plot(listXnp,listYnp,'ro',label='original')
    plt.hold(True)
    line2=plt.plot(listXnpFitted,listYnpFitted,'r-')
    line3=plt.plot(listXnp,listYnpCorrectted,'bo',label='corrected')
    line4=plt.plot(listXnpFitted,listYnpCorrecttedFitted,'b--')
    listXnpFitted,listYnpFitted,'r-',
    plt.hold(False) 
    
    plt.xlabel(Xlabel)
    plt.ylabel(Ylabel)

    plt.legend()

    plt.xlim(Left,Right)
    
    dummyBottom=min(min(listYnp),min(listYnpCorrectted))
    dummyTop=max(max(listYnp),max(listYnpCorrectted))
    Bottom=dummyBottom-0.2*(dummyTop-dummyBottom)
    Top=dummyTop+0.2*(dummyTop-dummyBottom)
    plt.ylim(Bottom,Top)

    plt.axvline(x=0,color='b',ls='dashed')
    
    #fittedEq = (Ylabel + " = " + str(slope) + " * " + Xlabel + " + " + str(intercept))
    #fittedEq = ("Y = (" + str(slpIntcpt[0]) + ") * X" + " + (" + str(slpIntcpt[1]) + ")")
    #fittedEq = ("Slope is " + str(round(slpIntcpt[0],2)) + 
    #            ". Intercept is " + str(round(slpIntcpt[1],2))+".")
    #fig.text(0.35, 0.25, fittedEq) 
    
    fig.text(0.14,0.25,("Slope: " + str(round(slpIntcpt[0],2))))       
    fig.text(0.14,0.21,("Intercept: " + str(round(slpIntcpt[1],2))))
    fig.text(0.14,0.17,("Correlation coefficent r: " + str(round(slpIntcpt[2],2))))       
    fig.text(0.14,0.13,("Standard deviation: " + str(round(slpIntcpt[3],2)))) 
    
    for i, dirName in enumerate(dirNames):
        fig.text(0.12+0.8*(listX[i]-Left)/(Right-Left)-0.03,
                 0.10+0.8*(listY[i]-Bottom)/(Top-Bottom)+0.01,dirName)       
    
    fig.suptitle(figTitle, fontsize=12)
                          
    plt.show()    

if __name__ == "__main__":
    
    #plotTitle='VO2+,O rich, mu_e=VBM'
    #perfectDir =  "../primordial_perfect/8x8x4_kmesh"
    #defectDirs = ["2x2x1","2x2x2","2x2x3","3x3x1","3x3x2","4x4x1"]
    #mu_e=6.268
    #mu_i={'Al':-3.74,'O':-4.20}

    plotTitle='VO2+,O rich, mu_e=VBM'
    perfectDir = "../../primordial_perfect/12x12x12_kmesh"
    defectDirs = ["1x2x2","1x2x3","1x3x4","2x2x2","2x3x4","3x3x3"]   
    mu_e=4.999
    mu_i={'Mg':-7.40,'O':-4.57}

    #plotTitle='VSi2-,C rich, mu_e=VBM'
    #perfectDir = "../primordial_perfect/12x12x12_kmesh"
    #defectDirs = ["2x2x2","2x2x3","2x3x3","3x3x3"]
    #defectDirs = ["2x2x2","2x2x3","2x2x4","2x3x3","3x3x3"]
    #mu_e=8.423
    #Si rich
    #mu_i={'Si':-5.44,'C':-9.65}
    #C rich
    #mu_i={'Si':-5.89,'C':-9.20}
    #Si/O equally rich
    #mu_i={'Si':-5.67,'C':-9.43}    
        
    saved = sys.stdout
    fout = file('out.log', 'w')
    sys.stdout = writer(sys.stdout, fout)
    
    cwdir=os.getcwd()
    os.chdir(defectDirs[0])    
    sc_struct=mg.io.vaspio.Poscar.from_file("CONTCAR").structure
    sc_potcar=mg.io.vaspio.Potcar.from_file('POTCAR')
    sc_incar=mg.io.vaspio.Incar.from_file('INCAR') 
    defchg=GenSC.defChg(sc_struct,sc_potcar,sc_incar)
    print "defect charge: ", defchg
    os.chdir(cwdir)
            
    (V_M,Eform_orig)=collectEformV_M(perfectDir,defectDirs,defchg,mu_e,mu_i)

    slpintcpt=linearFit(V_M,Eform_orig) 
    FS_corr=-np.array(V_M)*slpintcpt[0]
    Eform_corr=np.array(Eform_orig)+FS_corr

    print ("Folder " " V_M " "  EForm_orig" "  EForm_corrct " " FSCorrection")
    print ("-----------------------------------------------------")
    for j in range(len(defectDirs)):
        print (defectDirs[j]+"   "+
               str(round(V_M[j],2))+"   "+
               str(round(Eform_orig[j],5))+"    "+
               str(round(Eform_corr[j],5))+"       "+
               str(round(FS_corr[j],5)))    
    print ""       

    dielectConst=-defchg**2/2/slpintcpt[0]
    print "effective permittivity epsilon_fit: ", round(dielectConst,2)

    plotFit(V_M,Eform_orig,'Madelung Potential v_M (eV)','Formation Energy (eV)',slpintcpt,plotTitle,defectDirs)
    
    sys.stdout = saved
    fout.close()



import random
import numpy
from ase import Atom, Atoms

def atoms_add(indiv,Optimizer):
    """Move function to randomly add atoms or groups of atoms to a structure
    Input:
        indiv = individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Output:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    cell=indiv[0].get_cell()
    nato=indiv[0].get_number_of_atoms()
    xlen=cell[0][0]-cell[0][1]*cell[1][0]/cell[1][1]
    ylen=cell[1][1]-cell[1][0]*cell[0][1]/cell[0][0]
    zlen=cell[2][2]-cell[2][1]*cell[1][2]/cell[1][1]
    cell_min=[0,0,0]
    cell_max=[xlen,ylen,zlen]
    for i in range(random.randint(1,4)):
        atomsym=random.choice(Optimizer.atomlist)[0]
        pos=[random.uniform(cell_min[0],cell_max[0]), random.uniform(cell_min[1],cell_max[1]), random.uniform(cell_min[2],cell_max[2])]
        natom=Atom(atomsym,pos)
        indiv[0].append(natom)
    Optimizer.output.write('Add atom mutation\n')
    if Optimizer.structure=='Crystal':
        a, b, c = cell
        #normalized cell
        an, bn, cn = [numpy.linalg.norm(v) for v in cell]
        cell[0]=a/an
        cell[1]=b/bn
        cell[2]=c/cn
        indiv[0].set_cell(cell*[(indiv[0].get_number_of_atoms())**0.33*Optimizer.r_ab])
    muttype='AA'+repr(indiv[0].get_number_of_atoms()-nato)
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    Optimizer.output.write(repr(indiv[0].get_positions())+'\n')
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Wei Xie
# Last updated: 2014-02-07
##############################################################
import sys, getopt, os
import shutil
import matplotlib.pyplot as plt
import numpy as np
import scipy as sp

from pymatgen import Lattice, Structure
from pymatgen.io.smartio import read_structure, write_structure

stdX = read_structure("POSCAR_std")
prmX = read_structure("POSCAR_prm")
stdL = Structure.lattice(stdX)
prmL = Structure.lattice(stdX)


  
def finite_size_scale(standard, ssize, primordial, fsize, psize=[1,1,1]):
    """Function to perform finite size scaling for defect structure relaxation
    Inputs:
        standard = POSCAR file of structure containing defect
        ssize = Supercell size of structure with defect (list of size 3)
        primordial = POSCAR file of structure for basic unit of perfect cell
        psize = Supercell size of structure for padding. Default = [1,1,1] (list of size 3)
        fsize = Desired supercell size of final structure (list of size 3)
    Outputs:
        POSCAR file of structure containing defect and padding"""
    
    # Check if the input sizes work out with the desired final size
    padding = [0,0,0]
    for i in range(3):
        diff = fsize[i] - ssize[i]
        if diff < 0:
            raise RuntimeError('Desired final size of the structure must be larger than \
existing defect structure size. Defect Size = '+repr(ssize)+' Final Size = '+repr(fsize))
        elif diff >= 0:
             if math.fmod(diff,psize[i]):
                raise RuntimeError('Primordial structure and defect structure sizes cannot \
be used to form desired final size.  Reduce size of primordial structure. Defect Size = '+
    repr(ssize)+' Final Size = '+repr(fsize)+' Primordial size = '+repr(psize))
             else:
                padding[i] = diff/psize[i]
    
    # Load the defect structure and primordial structure
    try:
        defst = read_structure(standard)
    except:
        raise RuntimeError('Error: Unable to read standard structure.  Please check file. Filename: '+\
            standard)
    try:
        pst = read_structure(primordial)
    except:
        raise RuntimeError('Error: Unable to read primordial structure.  Please check file. Filename: '+\
            primordial)
    
    # Pad the structure
    positions = [site.coords for site in pst]
    syms = [str(site.specie.symbol) for site in pst]
    lv = [one/ssize for one in defst.lattice.matrix]
    vect = []
    for m0 in range(padding[0]):
        for m1 in numpy.arange(0,fsize[1],psize[1]):
            for m2 in numpy.arange(0,fsize[2],psize[2]):
                vect.append([ssize[0]+m0*psize[0],m1,m2])
    
    for m1 in range(padding[1]):
        for m0 in numpy.arange(0,ssize[0],psize[0]):
            for m2 in numpy.arange(0,fsize[2],psize[2]):
                vect.append([m0,ssize[1]+m1*psize[1],m2])
    
    for m2 in range(padding[2]):
        for m0 in numpy.arange(0,ssize[0],psize[0]):
            for m1 in numpy.arange(0,ssize[1],psize[1]):
                vect.append([m0,m1,ssize[2]+m2*psize[2]])
    
    #Construct a new structure with desired size
    new_lat = Lattice(numpy.array([fsize[c] * lv[c] for c in range(3)]))
    final = Structure(new_lat, defst.species_and_occu,defst.cart_coords,
            coords_are_cartesian=True)
    for m0,m1,m2 in vect:
        npos = positions + numpy.dot((m0, m1, m2), lv)
        for i in range(len(npos)):
            final.append(syms[i],npos[i],coords_are_cartesian=True)
    
    #Check for periodic issues in final structure
    final = check_periodic(final,defst)
    
    # Write output as POSCAR
    write_structure(final, 'POSCAR_Final')
    
    return final

def check_periodic(final,defect):
    # Identify nearest neighbor atoms for each atom in perfect structure
    #Identify trouble atoms
    natoms = len(defect)
    issueatoms = []
    for i in range(natoms):
        dist = [j for j in range(natoms,len(final)) 
            if i != j and final[i].distance(final[j])<=0.6]
        if len(dist) !=0:
            issueatoms.append(dist[0])
    #Idenfity nearest boundary and new coordinates
    coords = final.cart_coords
    for i in issueatoms:
        xflag = False
        yflag = False
        zflag = False
        scaledpos = numpy.linalg.solve(defect.lattice.matrix.T,final.cart_coords[i].T)
        relative = 0.1
        #Try reflection across x axis
        if 1-relative*0.1 < scaledpos[0] < 1+relative:
            npos = final.cart_coords[i]-numpy.dot(numpy.array([1,0,0]),defect.lattice.matrix)
            xflag = True
            coords[i][0] = npos[0]
        #Try reflection across y axis
        if 1-relative*0.1 < scaledpos[1] < 1+relative:
            npos = final.cart_coords[i]-numpy.dot(numpy.array([0,1,0]),defect.lattice.matrix)
            yflag = True
            coords[i][1] = npos[1]
        #Try reflection across z axis
        if 1-relative*0.1 < scaledpos[2] < 1+relative:
            npos = final.cart_coords[i]-numpy.dot(numpy.array([0,0,1]),defect.lattice.matrix)
            zflag = True
            coords[i][2] = npos[2]
        #print xflag, yflag, zflag
    #Generate new structure based on coordinates
    nfinal = Structure(final.lattice.matrix, final.species_and_occu, coords,
                coords_are_cartesian=True)
    return nfinal

# Potential issues:
#	--Poor accounting of internal periodicity  
#		-definition of relative term can be ambiguous depending on structure
#	--Assumes defect structure and primordial structures are aligned

if __name__ == "__main__":
    #Sanity checks	
    #Test 1 - Final size cannot be constructed directly from primordial structure
    ssize=[5,5,5]
    fsize=[9,5,5]
    psize=[2,5,5]
    felat = Lattice.cubic(2.87)
    primf = Structure(felat,["Fe","Fe"],[[0,0,0],[0.5,0.5,0.5]])
    primordialstructure = primf.copy()
    primordialstructure.make_supercell(psize)
    f = primf.copy()
    f.make_supercell(fsize)
    write_structure(primordialstructure,'POSCAR_prim1')
    final = finite_size_scale('POSCAR_Fdefect', ssize, 'POSCAR_prim1', fsize, psize)
    write_structure(f,'POSCAR_F1expected')
    write_structure(final,'POSCAR_F1')
    
    #Test 2 - Test in all 3 dimensions
    ssize=[5,5,5]
    fsize=[8,8,8]
    psize=[1,1,1]
    primordialstructure = primf.copy()
    primordialstructure.make_supercell(psize)
    f = primf.copy()
    f.make_supercell(fsize)
    write_structure(primordialstructure, 'POSCAR_prim2')
    final = finite_size_scale('POSCAR_Fdefect', ssize, 'POSCAR_prim2', fsize, psize)
    write_structure(f,'POSCAR_F2expected')
    write_structure(final,'POSCAR_F2')
    
    #Test 3 - Test for non-cubic lattice
    ssize = [4,4,4]
    fsize=[8,8,8]
    psize=[1,1,1]
    mglat = Lattice.hexagonal(3.21,5.21)
    primmg = Structure(mglat,["Mg","Mg"],[[1./3.,2./3.,3./4.],[2./3.,1./3.,1./4.]])
    primordialstructure = primmg.copy()
    primordialstructure.make_supercell(psize)
    f = primmg.copy()
    f.make_supercell(fsize)
    write_structure(primordialstructure,'POSCAR_prim3')
    final = finite_size_scale('POSCAR_Mdefect', ssize, 'POSCAR_prim3', fsize, psize)
    write_structure(f,'POSCAR_Mg3expected')
    write_structure(final,'POSCAR_Mg3')


import os
from ase import Atom, Atoms
from MAST.structopt.generate.defect import gen_solid
from MAST.structopt.tools.find_defects import find_defects
from MAST.structopt.generate.Individual import Individual
try:
	from mpi4py import MPI
except:
	pass

def get_defect_restart_indiv(Optimizer, indiv):
    """
    Function to generate an structopt Individual class object containing 
    	a defect structure from a previously existing structure
    Inputs:
    	Optimizer = structopt Optimizer class
    	indiv = ASE Atoms object containing the previously existing structure
    Outputs:
    	individ = structopt Individual class object containing defect structure data
    """
    if not Optimizer.solidbulk:
		#Initialize Bulk - Generate or load positions of bulk solid
		try:
		    rank = MPI.COMM_WORLD.Get_rank()
		except:
		    rank = 0
		outfilename = os.path.join(os.path.join(os.getcwd(),Optimizer.filename+'-rank'+repr(rank)),'Bulkfile.xyz')
		if Optimizer.evalsolid:
			bulk1, PureBulkEnpa, stro = gen_solid(Optimizer.solidfile,
				Optimizer.solidcell,outfilename,Optimizer.calc,Optimizer.calc_method)
			Optimizer.output.write(stro)
		else:
			bulk1 = gen_solid(Optimizer.solidfile,Optimizer.solidcell,outfilename)
			PureBulkEnpa = 0
		natomsbulk = len(bulk1)
		Optimizer.solidbulk = bulk1.copy()
		Optimizer.summary.write('CIBS Run Pure Bulk Energy per Atom:'+
			repr(PureBulkEnpa)+'\n')
		Optimizer.purebulkenpa = PureBulkEnpa
		Optimizer.natomsbulk = natomsbulk
    indiv.set_cell(Optimizer.solidcell)
    indiv.set_pbc(True)
    if Optimizer.restart_ints == 0:
        outt = find_defects(indiv,Optimizer.solidbulk,Optimizer.sf)
    else:
        indicop = [atm for atm in indiv if atm.symbol != 'X']
        indiv = Atoms(cell=Optimizer.solidcell, pbc=True)
        for atm in indicop:
            indiv.append(atm)
        outt=[indiv[0:Optimizer.restart_ints],indiv[Optimizer.restart_ints::], Atoms(),
        	Atoms(),'Assuming first '+repr(Optimizer.restart_ints)+' are interstitials\n']
    indi = outt[0].copy()
    bulki = outt[1].copy()
    individ = Individual(indi)
    individ.bulko = bulki.copy()
    individ.bulki = bulki.copy()
    individ.purebulkenpa = Optimizer.purebulkenpa
    individ.natomsbulk = Optimizer.natomsbulk
    individ.vacancies = outt[2].copy()
    individ.swaps = outt[3].copy()
    Optimizer.output.write(outt[4])
    return individ



from MAST.structopt.fingerprinting import fingerprint_dist
import random

def fussf(pop, nkeep, Optimizer):
	"""Selection function to employ basic Fixed Uniform Selection Scheme to select and 
	order individuals in a population based on fingerprint distance values.
	Picks one fuss point for the selection process
	FUSS point must reside within fusslimit
	"""
	fusslimit = Optimizer.fusslimit
	#FUSS for fingerprint
	newpop = []
	# Collect fitnesses
	fits = [ind.fitness for ind in pop]
	minindex = min(xrange(len(fits)), key=fits.__getitem__)
	fits = []
	STR = 'Distances: \n'
	for i in range(len(pop)):
		dist = fingerprint_dist(pop[minindex].fingerprint,pop[i].fingerprint)
		fits.append(dist)
		STR += repr(dist)+'\n'
	
	#Find min and max fitness
	minf = min(fits)
	maxf = max(fits)
	if abs(maxf-minf) > fusslimit:
		maxf = minf + fusslimit
	#Select random point on fitness line
	pt = random.uniform(minf,maxf)
	#Calculate the distance of each individual's fitness from that point
	i = 0
	distances = []
	for fit in fits:
		distances.append((abs(pt-fit),i))
		i += 1
	#Sort distances from min to max
	dlist = sorted(distances, key=lambda one: one[0])
	#Select individuals with lowest distance
	indices = []
	for i in range(nkeep):
		newpop.append(pop[dlist[i][1]])
		indices.append(dlist[i][1])
	#Always keep lowest energy individual
	if minindex not in indices:
		rm = random.choice(indices)
		newpop = [pop[inx] for inx in indices if inx != rm]
		newpop.append(pop[minindex])
	return newpop



from MAST.structopt.tools import get_best
import logging

def selection_switch(pop, nkeep, scheme, Optimizer):
    """Functions for selecting and pairing individuals 
    for crossovers"""
    #logger = initialize_logger(Optimizer.loggername)
    logger = logging.getLogger(Optimizer.loggername)
    if 'SEL' in Optimizer.debug:
        debug = True
    else:
        debug = False
    try:
        exec "from MAST.structopt.selection.{0} import {0}".format(scheme)
        newpop = eval('{0}(pop, nkeep, Optimizer)'.format(scheme))
    except NameError,e:
    	logger.warning('Selection scheme not one of the available options! Check Document and spelling. Selection Scheme : {0}. {1}'.format(scheme,e), exc_info=True)
    	logger.warning('No reordering or reduction applied')
    	newpop = pop
    except Exception, e:
    	logger.error('Issue in Selection scheme. Selection Scheme : {0}. {1}'.format(scheme,e), exc_info=True)
    	logger.warning('No reordering or reduction applied')
    	print 'ERROR: Issue in Selection scheme. Selection Scheme : '+repr(scheme)
        print e
        newpop = pop
    return newpop



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
# Purpose: Submit a job to the queue. This function is highly platform-specific and may need to be modified.
# Authors: Tam Mayeshiba
# 10/25/12 TTM created
# 12/6/12 TTM added UKY DLX commands
import os
import sys
from MAST.utility import dirutil
import subprocess
import time
from MAST.utility import MASTFile

def direct_shell_command():
    return "//share/apps/vasp5.2_cNEB"

def queue_submission_command():
    return "qsub"

def queue_status_from_text(jobid, queuetext):
    """
        Returns the queue status from a line with the job ID information
        INPUTS:
            jobid <int> = job ID
            qstr <str> = queue snapshot line containing job ID
        OUTPUTS:
            <str> = queue status 
            'E': Error
            'X': Not found
            'R': Running
            'Q': Queued
    """
    if (queuetext == None): 
        return 'X'
    if len(queuetext) == 0:
        return 'X'
    begin = queuetext.find(str(jobid))
    if begin == -1:
        return 'X'
    queuetext = queuetext[begin:]
    end = queuetext.find('\n')
    if end > -1:
        queuetext = queuetext[:end]
    
    qmat = queuetext.split()
    if len(qmat) < 5:
        return 'E'
    return qmat[4] #TTM 1/17/12 qstat returns differently than qstat -a does

    #if system == "ranger":####Ranger
    #    ranger_out = queuetext.split()[4]
    #    if (ranger_out == 'wq') or (ranger_out == 'qw'):
    #        final_out = 'Q'
    #    elif ranger_out == 'r':
    #        final_out = 'R'
    #    return final_out

    #if system == "dlx":####UKy DLX
    #    queuetext = queuetext.strip()
    #    dlx_out = queuetext.split()[4] #indexing starts at 0
    #    if (dlx_out == 'PD'):
    #        final_out = 'Q'
    #    elif dlx_out in ['CG','CD','R']:
    #        final_out = 'R'
    #    return final_out

def extract_submitted_jobid(string):
    """
        Extract the job ID from a string returned when the job is submitted
        INPUTS:
            string <str> = string with job ID somewhere in it
        OUTPUTS:
            <int> = job ID as integer
    """
    if string == "":
        return None
    return int(string.split('.')[0])
    ####Bardeen and Curie:
    if system == "bardeen" or system == "curie":
        final = ""
        for char in string:
            if(char != '.'):
                final += char
            else:
                return int(final)

    if system == "ranger":####Ranger:
        findme=0
        findstr=""
        findme = string.find("Your job") # ..... Your job 12345678 has been submitted.
        if findme == -1: #TTM+1 2/11/12 if not found, return negative 1
            return -1
        findstr = string[findme+9:]
        splitstr=[]
        splitstr = findstr.split()
        return int(splitstr[0])

    if system == "dlx":###UKY DLX:
        findstr = string.strip()
        return int(findstr.split()[3])

def queue_snap_command():
    """
        Return the command for producing a queue snapshot.
        INPUTS:
            None
        OUTPUTS:
            Command for producing a queue snapshot
    """
    return "qstat"

    qcmd=""
    if(compute_node):
        if(system == "ranger"):####Ranger, from compute node
            #
            qcmd = "ssh -f ranger 'qstat'"

        if(system == "bardeen"):####Bardeen, from compute node
            qcmd = "ssh -f bardeen 'qstat'" 
    
        if(system == "curie"):####Curie, from compute node
            qcmd = "ssh -f curie 'qstat'"
    else:###Direct from headnode
        qcmd = 'qstat 2> /dev/null' #TTM 052212 sometimes requires /bin/bash
    ###Direct from headnode, UKY DLX:
        if(system == "dlx"):
            uname=os.getenv("USER")
            qcmd = "squeue -u " + uname
    return qcmd

def get_approx_job_error_file(jobid):
    """
        Return the approximate job error file name
            Args:
                jobid <int>: Job ID
    """
    if jobid == None:
        return ".e"
    else:
        return ".e%s" % str(jobid)



import random
from ase import Atom, Atoms

def cxtpa(ind1, ind2, Optimizer):
    """ Simple Two point Crossover of atoms
    Maintains concentration
    Maintains total number of atoms
    Random clusters not necessarily geometry based
    """
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    # Get individual atoms
    Optimizer.output.write('Two Pt Atom Cx between individual '+repr(ind1.index)+' and individual '+repr(ind2.index)+'\n')
    indi1=ind1[0]
    indi2=ind2[0]
    #Identify range of atoms in individual 1 to exchange
    cxpoint1 = random.randint(0, len(indi1)-1)
    cxpoint2 = random.randint(1, len(indi1)-1)
    if cxpoint2 == cxpoint1:
        cxpoint2 += 1
    elif cxpoint2 < cxpoint1:
        # Swap the two cx points
        cxpoint1, cxpoint2 = cxpoint2, cxpoint1
    group1 = Atoms()
    symlist = []
    indices1 = []
    for one in range(cxpoint1, cxpoint2):
        group1.append(indi1[one])
        indices1.append(one)
        if indi1[one].symbol not in symlist:
            symlist.append(indi1[one].symbol)
    #Identify atoms in individual 2 of similar structure as in individual 1
    group2 = Atoms()
    seplist = [[atm for atm in indi2 if atm.symbol == sym] for sym in symlist]
    indices2 = []
    dellist = []
    for one in group1:
        sym1=one.symbol
        listpos=[i for i,value in enumerate(symlist) if value==sym1][0]
        if len(seplist[listpos]) > 0:
            pos = random.choice(range(len(seplist[listpos])))
            group2.append(seplist[listpos][pos])
            indices2.append(seplist[listpos][pos].index)
            del seplist[listpos][pos]
        else:
            dellist.append(one.index)
    if len(dellist) != 0:
        dellist.sort(reverse=True)
        for one in dellist:
            del group1[one]
            del indices1[one]
    #Identify atoms not to be exchanged
    other1=Atoms()
    for one in indi1:
        if one.index not in indices1:
            other1.append(one)
    other2 = Atoms()
    for one in indi2:
        if one.index not in indices2:
            other2.append(one)
    #Perform exchange
    indi1n = other1.copy()
    indi1n.extend(group2)
    indi2n = other2.copy()
    indi2n.extend(group1)
    #Perserve cell and boundary conditions
    indi1n.set_cell(indi1.get_cell())
    indi1n.set_pbc(indi1.get_pbc())
    indi2n.set_cell(indi2.get_cell())
    indi2n.set_pbc(indi2.get_pbc())
    ind1[0]=indi1n
    ind2[0]=indi2n
    #Debug options
    if debug:
        print 'DEBUG CX: Number of atoms exchanged from Individual 1 = ', len(group1)
        print 'DEBUG CX: Number of atoms exchanged from Individual 2 = ', len(group2)
        for s,c,m,u in Optimizer.atomlist:
            print 'DEBUG CX: Individual 1 has ', len([atm for atm in indi1n if atm.symbol==s]), ' atoms of structure ', s
            print 'DEBUG CX: Individual 2 has ', len([atm for atm in indi2n if atm.symbol==s]), ' atoms of structure ', s
        print 'DEBUG CX: Number of atoms in Individual 1 = ', len(indi1n)
        print 'DEBUG CX: Number of atoms in Individual 2 = ', len(indi2n)
    return ind1, ind2



from ase.calculators.neighborlist import NeighborList
from scipy.stats import mode
from ase import Atoms, Atom
import copy
import random
import numpy
import math
from MAST.structopt.tools.calc_dist import calc_dist
from MAST.structopt.tools.eval_energy import run_energy_eval as REE

def fss_bcc(indiv, Optimizer):
    defected = indiv[0].copy()
    defected.extend(indiv.bulki)
    #Identify nearest neighbor cutoff distance
    nndists = []
    for i in range(5):
        one = random.choice(defected)
        distances = [defected.get_distance(one.index, j) for j in range(len(defected)) if j != one.index]
        distances.sort()
        nndists.extend(distances[0:3])
    nndist = sum(nndists)/len(nndists)
    cutoff = nndist*0.6
    #Create nearest neighbor list from cutoff distance
    ctflist = [cutoff for one in defected]
    nl = NeighborList(ctflist, bothways=True, self_interaction=False)
    nl.update(defected)
    #Identify most common number of nearest neighbors for each atom
    nneigh = []
    for one in defected:
        indices, offsets = nl.get_neighbors(one.index)
        nneigh.append(len(indices))
    avn = mode(nneigh)
    #Identify those atoms that have a different number of nearest neighbors
    defs = [i for i in range(len(nneigh)) if nneigh[i] != avn[0][0]]
    #Create new structure from translated defected atoms
    defsat = Atoms(pbc = defected.get_pbc(), cell=defected.get_cell())
    for i in defs:
        defsat.append(defected[i])
    #Identify center of mass of defected group and translate to center of cell
    cop = position_average(defsat)
    ndefsat = shift_atoms(defsat, cop)
    ndefected = shift_atoms(defected, cop)
    #Identify bounds of defected portion of structure
    maxpos = max(numpy.maximum.reduce(ndefsat.positions))
    minpos = min(numpy.minimum.reduce(ndefsat.positions))
    #Identify size of structure that will encompass defected structure
    osc = copy.deepcopy(Optimizer.supercell)
    latcont = numpy.maximum.reduce(ndefsat.get_cell())[0]/osc[0]
    deltapos = abs(maxpos-minpos)
    newsupercell = round(deltapos/latcont)+1
    bxlen = newsupercell*latcont
    #Identify those atoms within a box of encompassing length centered at the center of the cell
    tol = .1
    cell = numpy.maximum.reduce(ndefsat.get_cell())
    boxlow = [cell[i]/2.0-bxlen/2.0-tol for i in range(3)]
    boxhigh = [cell[i]/2.0+bxlen/2.0+tol for i in range(3)]
    atlist = []
    otherlist = []
    for one in ndefected:
        pos = one.position
        if boxlow[0] < pos[0] < boxhigh[0]:
            if boxlow[1] < pos[1] < boxhigh[1]:
                if boxlow[2] < pos[2] < boxhigh[2]:
                    atlist.append(one)
                else:
                    otherlist.append(one)
            else:
                otherlist.append(one)
        else:
            otherlist.append(one)
    ncell = [bxlen,bxlen,bxlen]
    #Create a new atoms object from the atoms within the box
    ssats = Atoms(pbc = True, cell=ncell)
    for one in atlist:
        ssats.append(one)
    #Attach a calculator for the atoms
    ssats.set_calculator(Optimizer.calc)
    #Calculate the energy
    out = REE(ssats)

totalsol, energy, pressure, volume, STR

    return defsat  

def position_average(atoms, mode='median', trimp = 0.05):
    """Function to calculate the trimmed mean or median
    of the positions of atoms
    Input:
        atoms = ASE Atoms object
        mode = method to use either median or trimmedmean
            default value is median
        trimp = percentage of high and lows to leave out
            default value is 5% == 0.05
    Output:
        [x,y,z] = position of atoms center
    """
    positions = atoms.get_positions()
    xs = []
    ys = []
    zs = []
    for x,y,z in positions:
        xs.append(x)
        ys.append(y)
        zs.append(z)
    xs.sort()
    ys.sort()
    zs.sort()
    if mode=='median':
        xc = xs[len(xs)/2]
        yc = ys[len(ys)/2]
        zc = zs[len(zs)/2]
    else:
        ndel = int(round(trimp*len(xs)))
        xc = sum(xs[ndel:len(xs)-ndel])/(len(xs)-2*ndel)
        yc = sum(ys[ndel:len(ys)-ndel])/(len(ys)-2*ndel)
        zc = sum(zs[ndel:len(zs)-ndel])/(len(zs)-2*ndel)
    return [xc,yc,zc]

def shift_atoms(atoms, position=None):
    """Function to shift the positions of the atoms so that
    the given location is in the center of the cell.
    Input:
        atoms = ASE atoms object to be shifted
        position = [x,y,z] position to shift to center of cell
            default is center of mass of atoms
    Output:
        atoms = ASE atoms object that has been shifted
    """
    atoms = atoms.copy()
    if not position:
        position = atoms.get_center_of_mass
    cell = numpy.maximum.reduce(atoms.get_cell())
    trans = [cell[i]/2.0-position[i] for i in range(3)]
    atoms.translate(trans)
    positions = atoms.get_positions()
    for i in range(len(positions)):
        if atoms.pbc[0]:
            if trans[0] > 0:
                if positions[i][0] > cell[0]:
                    positions[i][0] -= cell[0]
            else:
                if positions[i][0] < 0:
                    positions[i][0] += cell[0]
        if atoms.pbc[1]:
            if trans[1] > 0:
                if positions[i][1] > cell[1]:
                    positions[i][1] -= cell[1]
            else:
                if positions[i][1] < 0:
                    positions[i][1] += cell[1]
        if atoms.pbc[2]:
            if trans[2] >0:
                if positions[i][2] > cell[2]:
                    positions[i][2] -= cell[2]
            else:
                if positions[i][2] < 0:
                    positions[i][2] += cell[2]
    atoms.set_positions(positions)
    return atoms

